@@ -53,7 +53,7 @@ class Assets(WMLResource):
         Assets._validate_type(asset_uid, u'asset_uid', STR_TYPE, True)
 
 
-        if not self._ICP:
+        if not self._ICP and not self._client.WSD:
             response = requests.get(self._href_definitions.get_data_asset_href(asset_uid), params=self._client._params(),
                                     headers=self._client._get_headers())
         else:
@@ -336,7 +336,7 @@ class Assets(WMLResource):
         if limit is not None:
             data.update({"limit": limit})
 
-        if not self._ICP:
+        if not self._ICP and not self._client.WSD:
             response = requests.post(href, params=self._client._params(), headers=self._client._get_headers(),json=data)
         else:
             response = requests.post(href, params=self._client._params(), headers=self._client._get_headers(),json=data, verify=False)
@@ -375,7 +375,7 @@ class Assets(WMLResource):
         Assets._validate_type(asset_uid, u'asset_uid', STR_TYPE, True)
 
         import urllib
-        if not self._ICP:
+        if not self._ICP and not self._client.WSD:
             asset_response = requests.get(self._href_definitions.get_data_asset_href(asset_uid),
                                           params=self._client._params(),
                                           headers=self._client._get_headers())
@@ -405,7 +405,7 @@ class Assets(WMLResource):
                 raise WMLClientError(u'Saving data asset with artifact_url: \'{}\'  to local file failed.'.format(filename), e)
         else:
             attachment_id = asset_details["attachments"][0]["id"]
-            if not self._ICP:
+            if not self._ICP and not self._client.WSD:
                 response = requests.get(self._href_definitions.get_attachment_href(asset_uid,attachment_id), params=self._client._params(),
                                         headers=self._client._get_headers())
             else:
@@ -430,7 +430,7 @@ class Assets(WMLResource):
                          It is supported only for Cloud storage data source type connections ')
                 else:
                     attachment_signed_url = response.json()["url"]
-                    if not self._ICP:
+                    if not self._ICP and not self._client.WSD:
                         att_response = requests.get(self._wml_credentials["url"]+attachment_signed_url)
                     else:
                         att_response = requests.get(self._wml_credentials["url"]+attachment_signed_url,
@@ -534,7 +534,7 @@ class Assets(WMLResource):
         """
         Assets._validate_type(asset_uid, u'asset_uid', STR_TYPE, True)
 
-        if not self._ICP:
+        if not self._ICP and not self._client.WSD:
             response = requests.delete(self._href_definitions.get_asset_href(asset_uid), params=self._client._params(),
                                     headers=self._client._get_headers())
         else:
@@ -549,7 +549,7 @@ class Assets(WMLResource):
     def _delete(self, asset_uid):
         Assets._validate_type(asset_uid, u'asset_uid', STR_TYPE, True)
 
-        if not self._ICP:
+        if not self._ICP and not self._client.WSD:
             response = requests.delete(self._href_definitions.get_asset_href(asset_uid), params=self._client._params(),
                                        headers=self._client._get_headers())
         else:
@@ -616,7 +616,7 @@ class Assets(WMLResource):
                                        'href': response_data['href'],
                                        'asset_type': response_data['metadata']['asset_type'],
                                        'created_at': response_data['metadata']['created_at'],
-                                       'last_updated_at': response_data['metadata']['usage']['last_updated_at']
+                                       'last_updated_at': response_data['metadata']['usage'].get('last_updated_at')
                                        },
                           'entity': response_data['entity']
 
@@ -631,7 +631,7 @@ class Assets(WMLResource):
                                            'href': href,
                                            'asset_type': response_data['metadata']['asset_type'],
                                            'created_at': response_data['metadata']['created_at'],
-                                           'last_updated_at': response_data['metadata']['usage']['last_updated_at']
+                                           'last_updated_at': response_data['metadata']['usage'].get('last_updated_at')
                                            },
                               'entity': response_data['entity']
 
@@ -642,7 +642,7 @@ class Assets(WMLResource):
                                        'href': response_data['href'],
                                        'asset_type': response_data['metadata']['asset_type'],
                                        'created_at': response_data['metadata']['created_at'],
-                                       'last_updated_at': response_data['metadata']['usage']['last_updated_at']
+                                       'last_updated_at': response_data['metadata']['usage'].get('last_updated_at')
                                        },
                              'entity': response_data['entity']
 
