@@ -52,6 +52,7 @@ class WatsonMachineLearningAPIClient:
         self.wml_credentials = copy.deepcopy(wml_credentials)
         self.CAMS = None
         self.WSD = None
+        self.WSD_20 = None
         self.ICP_30 = None
         self.ICP = None
         self.default_space_id = None
@@ -78,8 +79,10 @@ class WatsonMachineLearningAPIClient:
                     url_port = wml_credentials[u'url'].split(':')[-1]
                     if not url_port.isdigit():
                         raise WMLClientError("It is mandatory to have port number as part of url for wml_local.")
-                if '3.0.0' == wml_credentials[u'version'].lower() or '3.0.1' == wml_credentials[u'version'].lower()\
-                        or '2.0' == wml_credentials[u'version'].lower():
+
+                if '3.0.0' == wml_credentials[u'version'].lower() or \
+                   '3.0.1' == wml_credentials[u'version'].lower() or \
+                   '2.0' == wml_credentials[u'version'].lower():
                     self.ICP_30 = True
 
             else:
@@ -95,6 +98,8 @@ class WatsonMachineLearningAPIClient:
                ('1.1' == wml_credentials[u'version'].lower() or '2.0' == wml_credentials[u'version'].lower()):
                 self.WSD = True
                 os.environ['WSD_PLATFORM'] = 'True'
+                if '2.0' == wml_credentials[u'version'].lower():
+                    self.WSD_20 = True
             else:
                 if ('wsd_local' == wml_credentials[u'instance_id'].lower()) and\
                    'version' in wml_credentials.keys() and \
@@ -135,6 +140,8 @@ class WatsonMachineLearningAPIClient:
                 self.script = Script(self)
                 if not '2.0' == wml_credentials[u'version'].lower():
                     self.shiny = Shiny(self)
+            if self.WSD_20:
+                self.software_specifications = SwSpec(self)
 
         #    self.learning_system = LearningSystem(self)
         self.repository = Repository(self)
@@ -179,13 +186,19 @@ class WatsonMachineLearningAPIClient:
                     "It is mandatory to set the project id. Use client.set.default_project(<PROJECT_UID>) to proceed.")
         return params
 
-    def _get_headers(self, content_type='application/json', no_content_type=False):
+    def _get_headers(self, content_type='application/json', no_content_type=False, wsdconnection_api_req=False):
         if self.WSD:
                 headers = {'X-WML-User-Client': 'PythonClient'}
                 if self.project_id is not None:
                     headers.update({'X-Watson-Project-ID': self.project_id})
                 if not no_content_type:
                     headers.update({'Content-Type': content_type})
+                if wsdconnection_api_req is True:
+                    token = "desktop user token"
+                else:
+                    token = "desktop-token"
+                headers.update({'Authorization':  "Bearer " + token})
+
         else:
             if self.proceed is True:
                 token = "Bearer "+ self.wml_credentials["token"]
