@@ -12,10 +12,12 @@ from pandas import DataFrame
 from pandas import Series
 from watson_machine_learning_client.utils.autoai.enums import (
     PredictionType, Directions, MetricsToDirections, PipelineTypes)
-from watson_machine_learning_client.utils.autoai.errors import FitNeeded
+from watson_machine_learning_client.utils.autoai.errors import FitNeeded, MissingDataPreprocessingStep
 from watson_machine_learning_client.utils.autoai.local_training_message_handler import LocalTrainingMessageHandler
 from watson_machine_learning_client.utils.autoai.utils import (
-    try_import_lale, create_summary, download_experiment_details_from_file, prepare_model_location_path)
+    try_import_lale, create_summary, download_experiment_details_from_file, prepare_model_location_path,
+    download_wml_pipeline_details_from_file)
+from watson_machine_learning_client.preprocessing import DataJoinGraph, DataJoinPipeline
 
 from .base_auto_pipelines import BaseAutoPipelines
 
@@ -95,6 +97,7 @@ class LocalAutoPipelines(BaseAutoPipelines):
 
         self._force_local_scenario = _force_local_scenario
         self._training_data_reference = None
+        self._training_result_reference = None
 
         # note: Local scenario should be implemented in the future (ai4ml needed locally)
         if _data_clients is None and _result_client is None:
@@ -186,6 +189,16 @@ class LocalAutoPipelines(BaseAutoPipelines):
                                                 "XGBClassifierEstimator"]
             }
         """
+        if hasattr(self, '_result_client'):
+            wml_pipeline_details = download_wml_pipeline_details_from_file(self._result_client)
+
+            # note: check if we have obm preprocessing step
+            if 'auto_ai.obm' in str(wml_pipeline_details):
+                obm_node = wml_pipeline_details['pipelines'][0]['nodes'][0]
+                data_join_graph = DataJoinGraph()._from_dict(_dict=obm_node)
+                self.params['data_join_graph'] = data_join_graph
+                # --- end note
+
         return self.params
 
     def fit(self, X: 'DataFrame', y: 'Series') -> 'Pipeline':
@@ -300,8 +313,9 @@ class LocalAutoPipelines(BaseAutoPipelines):
 
             if (MetricsToDirections[self._pdcoe.scorer_for_ranking.upper()].value ==
                     Directions.ASCENDING):
-                return pipelines.sort_values(by=[f"training_{self._pdcoe.scorer_for_ranking}"], ascending=False).rename({
-                    f"training_{self._pdcoe.scorer_for_ranking}": f"training_{self._pdcoe.scorer_for_ranking}_(optimized)"},
+                return pipelines.sort_values(by=[f"training_{self._pdcoe.scorer_for_ranking}"], ascending=False).rename(
+                    {
+                        f"training_{self._pdcoe.scorer_for_ranking}": f"training_{self._pdcoe.scorer_for_ranking}_(optimized)"},
                     axis='columns')
 
             else:
@@ -344,7 +358,8 @@ class LocalAutoPipelines(BaseAutoPipelines):
 
             for pipeline in details['entity']['status'].get('metrics', []):
                 if (pipeline['context']['phase'] == 'global_output' and
-                        pipeline['context']['intermediate_model']['name'].split('P')[-1] == pipeline_name.split('_')[-1]):
+                        pipeline['context']['intermediate_model']['name'].split('P')[-1] == pipeline_name.split('_')[
+                            -1]):
                     pipeline_parameters['composition_steps'] = pipeline['context']['intermediate_model'][
                         'composition_steps']
                     pipeline_parameters['pipeline_nodes'] = pipeline['context']['intermediate_model']['pipeline_nodes']
@@ -367,7 +382,8 @@ class LocalAutoPipelines(BaseAutoPipelines):
             pipeline_parameters = {
                 "composition_steps": self._computed_pipelines_details[pipeline_name]['CompositionSteps'].values(),
                 "pipeline_nodes": [node['op'] for node in
-                                   self._computed_pipelines_details[pipeline_name]['Params']['pipeline']['nodes'].values()],
+                                   self._computed_pipelines_details[pipeline_name]['Params']['pipeline'][
+                                       'nodes'].values()],
             }
 
             return pipeline_parameters
@@ -538,6 +554,58 @@ class LocalAutoPipelines(BaseAutoPipelines):
         else:
             raise NotImplementedError("Local scenario not yet implemented.")
 
+    def get_preprocessed_data_connection(self) -> 'DataConnection':
+        """
+        Provides DataConnection with preprocessed training data.
+
+        Returns
+        -------
+        DataConnection with populated optimizer parameters
+        """
+        if hasattr(self, '_result_client'):
+            params = self.get_params()
+
+            if params.get('data_join_graph'):
+                details = download_experiment_details_from_file(self._result_client)
+
+                path = details['entity']['status']['metrics'][0]['context']['intermediate_model'][
+                    'location']['model'].split('/data/kb/')[0]
+                path = f"{path}/data/obm/features/part"
+
+                self._training_result_reference._obm = True  # indicator for OBM output data
+                self._training_result_reference._obm_cos_path = path
+
+                return self._training_result_reference
+
+            else:
+                raise MissingDataPreprocessingStep(
+                    reason="Cannot get preprocessed pipeline as preprocessing step was not performed.")
+
+        else:
+            raise NotImplementedError("Local scenario not yet implemented.")
+
+    def get_preprocessing_pipeline(self) -> 'DataJoinPipeline':
+        """
+        Returns preprocessing pipeline object for further usage.
+            (eg. to visualize preprocessing pipeline as graph).
+
+        Returns
+        -------
+        DataJoinPipeline
+        """
+        if hasattr(self, '_result_client'):
+            optimizer_parameters = self.get_params()
+
+            if optimizer_parameters['data_join_graph']:
+                return DataJoinPipeline(self.get_preprocessed_data_connection())
+
+            else:
+                raise MissingDataPreprocessingStep(
+                    reason="Cannot get preprocessed pipeline as preprocessing step was not performed.")
+
+        else:
+            raise NotImplementedError("Local scenario not yet implemented.")
+
     def _train(self, train_x: 'DataFrame', train_y: 'Series') -> 'PrepDaubCogOptEstimator':
         """
         Prepare and run PDCOE optimizer/estimator.
