@@ -1,33 +1,42 @@
-import io
-import uuid
-from copy import deepcopy
-from typing import Union, Tuple, List
-
-import requests
-from ibm_boto3 import resource
-from pandas import DataFrame
-from watson_machine_learning_client.utils.autoai.enums import PredictionType, DataConnectionTypes
-from watson_machine_learning_client.utils.autoai.errors import (
-    MissingAutoPipelinesParameters, UseWMLClient, MissingCOSStudioConnection, MissingProjectLib,
-    HoldoutSplitNotSupported, InvalidCOSCredentials
-)
-from watson_machine_learning_client.utils.autoai.utils import try_load_dataset
-from watson_machine_learning_client.wml_client_error import ApiRequestFailure
-
-from .base_connection import BaseConnection
-from .base_data_connection import BaseDataConnection
-from .base_location import BaseLocation
-
 __all__ = [
     "DataConnection",
     "S3Connection",
     "S3Location",
     "FSLocation",
-    "DSLocation"
+    "DSLocation",
+    "CP4DAssetLocation",
+    "WMLSAssetLocation",
+    "WSDAssetLocation",
+    "DeploymentOutputAssetLocation"
 ]
 
+import io
+import os
+import uuid
+from copy import deepcopy
+from typing import Union, Tuple, List, TYPE_CHECKING, Optional
+from warnings import warn
+
+from ibm_boto3 import resource
+from ibm_botocore.client import ClientError
+from pandas import DataFrame
+import requests
+
+from watson_machine_learning_client.utils.autoai.enums import PredictionType, DataConnectionTypes
+from watson_machine_learning_client.utils.autoai.errors import (
+    MissingAutoPipelinesParameters, UseWMLClient, MissingCOSStudioConnection, MissingProjectLib,
+    HoldoutSplitNotSupported, InvalidCOSCredentials, MissingLocalAsset
+)
+from watson_machine_learning_client.utils.autoai.watson_studio import get_project
+from watson_machine_learning_client.wml_client_error import MissingValue, ApiRequestFailure
+from .base_connection import BaseConnection
+from .base_data_connection import BaseDataConnection
+from .base_location import BaseLocation
+
+if TYPE_CHECKING:
+    from watson_machine_learning_client.workspace import WorkSpace
+
 
-# TODO: read/write support only for small files, need to implement large files support
 class DataConnection(BaseDataConnection):
     """
     Data Storage Connection class needed for WML training metadata (input data).
@@ -37,13 +46,20 @@ class DataConnection(BaseDataConnection):
     connection: Union[S3Connection], required
         connection parameters of specific type
 
-    location: Union[S3Location], required
+    location: Union[S3Location, FSLocation, CP4DAssetLocation, WMLSAssetLocation, WSDAssetLocation], required
         location parameters of specific type
     """
 
     def __init__(self,
-                 location: Union['S3Location', 'FSLocation', 'DSLocation'],
-                 connection: Union['S3Connection'] = None):
+                 location: Union['S3Location',
+                                 'FSLocation',
+                                 'CP4DAssetLocation',
+                                 'WMLSAssetLocation',
+                                 'WSDAssetLocation',
+                                 'DeploymentOutputAssetLocation'],
+                 connection: Optional['S3Connection'] = None,
+                 data_join_node_name: str = None):
+        super().__init__()
 
         if isinstance(connection, S3Connection):
             self.type = DataConnectionTypes.S3
@@ -51,15 +67,29 @@ class DataConnection(BaseDataConnection):
         elif isinstance(location, FSLocation):
             self.type = DataConnectionTypes.FS
 
-        elif isinstance(location, DSLocation):
+        elif isinstance(location, (CP4DAssetLocation, WMLSAssetLocation, WSDAssetLocation, DeploymentOutputAssetLocation)):
             self.type = DataConnectionTypes.DS
 
         self.connection = connection
         self.location = location
 
-        self.auto_pipeline_params = None  # note: needed parameters for recreation of autoai holdout split
+        self.auto_pipeline_params = {}  # note: needed parameters for recreation of autoai holdout split
         self._wml_client = None
         self._run_id = None
+        self._obm = False
+        self._obm_cos_path = None
+
+        # note: make data connection id as a location path for OBM + KB
+        if data_join_node_name is None:
+            if self.type == DataConnectionTypes.S3:
+                self.id = location.path
+
+            else:
+                self.id = None
+
+        else:
+            self.id = data_join_node_name
+        # --- end note
 
     @classmethod
     def from_studio(cls, path: str) -> List['DataConnection']:
@@ -125,6 +155,11 @@ class DataConnection(BaseDataConnection):
         """
         _dict = {"type": self.type}
 
+        # note: for OBM (id of DataConnection if an OBM node name)
+        if self.id is not None:
+            _dict['id'] = self.id
+        # --- end note
+
         if self.connection is not None:
             _dict['connection'] = deepcopy(self.connection.to_dict())
 
@@ -172,26 +207,25 @@ class DataConnection(BaseDataConnection):
             )
 
         else:
+            # note: recognizing environment by location href
+            if 'projet_id=' in _dict['location']['href']:
+                asset_location = CP4DAssetLocation._set_path(href=_dict['location']['href'])
+            else:
+                asset_location = WMLSAssetLocation._set_path(href=_dict['location']['href'])
+            # --- end note
             data_connection: 'DataConnection' = cls(
-                location=DSLocation._set_path(href=_dict['location']['href'])
+                location=asset_location
             )
 
+        if _dict.get('id'):
+            data_connection.id = _dict['id']
+
         return data_connection
 
-    def _fill_experiment_parameters(self, prediction_type: str, prediction_column: str, test_size: float,
-                                    csv_separator: str = ',', excel_sheet: Union[str, int] = 0) -> None:
-        """
-        To be able to recreate a holdout split, this method need to be called.
-        """
-        self.auto_pipeline_params = {
-            'prediction_type': prediction_type,
-            'prediction_column': prediction_column,
-            'test_size': test_size,
-            'csv_separator': csv_separator,
-            'excel_sheet': excel_sheet
-        }
-
-    def read(self, with_holdout_split: bool = False) -> Union['DataFrame', Tuple['DataFrame', 'DataFrame']]:
+    def read(self,
+             with_holdout_split: bool = False,
+             csv_separator: str = ',',
+             excel_sheet: Union[str, int] = 0) -> Union['DataFrame', Tuple['DataFrame', 'DataFrame']]:
         """
         Download dataset stored in remote data storage.
 
@@ -200,6 +234,12 @@ class DataConnection(BaseDataConnection):
         with_holdout_split: bool, optional
             If True, data will be split to train and holdout dataset as it was by AutoAI.
 
+        csv_separator: str, optional
+            Separator / delimiter for CSV file, default is ','
+
+        excel_sheet: Union[str, int], optional
+            Excel file sheet name to use, default is 0.
+
         Returns
         -------
         pandas.DataFrame contains dataset from remote data storage or Tuple[pandas.DataFrame, pandas.DataFrame]
@@ -207,25 +247,35 @@ class DataConnection(BaseDataConnection):
             (only if only_holdout == True and auto_pipeline_params was passed)
         """
 
+        if with_holdout_split:
+            print("\"with_holdout_split\" option is depreciated.")
+            warn("\"with_holdout_split\" option is depreciated.")
+
         from sklearn.model_selection import train_test_split
 
-        if with_holdout_split and self.auto_pipeline_params is None:
+        if with_holdout_split and not self.auto_pipeline_params.get('prediction_type', False):
             raise MissingAutoPipelinesParameters(
                 self.auto_pipeline_params,
                 reason=f"To be able to recreate an original holdout split, you need to schedule a training job or "
                        f"if you are using historical runs, just call historical_optimizer.get_data_connections()")
 
+        # note: allow to read data at any time
+        elif (('csv_separator' not in self.auto_pipeline_params and 'excel_sheet' not in self.auto_pipeline_params)
+              or csv_separator != ',' or excel_sheet != 0):
+            self.auto_pipeline_params['csv_separator'] = csv_separator
+            self.auto_pipeline_params['excel_sheet'] = excel_sheet
+        # --- end note
+
         data = DataFrame()
         if self.type == DataConnectionTypes.S3:
             cos_client = self._init_cos_client()
 
             try:
-                file = cos_client.Object(self.location.bucket,
-                                         self.location.path).get()
-                buffer = io.BytesIO(file['Body'].read())
-                data = try_load_dataset(buffer=buffer,
-                                        sheet_name=self.auto_pipeline_params.get('excel_sheet', 0),
-                                        separator=self.auto_pipeline_params.get('csv_separator', ','))
+                if self._obm:
+                    data = self._download_obm_data_from_cos(cos_client=cos_client)
+
+                else:
+                    data = self._download_data_from_cos(cos_client=cos_client)
 
             except Exception as cos_access_exception:
                 raise ConnectionError(
@@ -233,46 +283,19 @@ class DataConnection(BaseDataConnection):
                     f"Error: {cos_access_exception}")
 
         elif self.type == DataConnectionTypes.DS:
-            # note: as we need to load a data into the memory,
-            # we are using pure requests and helpers from the WML client
-            asset_id = self.location.href.split('?')[0].split('/')[-1]
 
-            # note: download data asset details
-            asset_response = requests.get(self._wml_client.data_assets._href_definitions.get_data_asset_href(asset_id),
-                                          params=self._wml_client._params(),
-                                          headers=self._wml_client._get_headers(),
-                                          verify=False)
+            if self._obm:
+                raise NotImplementedError("Multiple files for CP4D / WML Server is not yet supported.")
 
-            asset_details = self._wml_client.data_assets._handle_response(200, u'get assets', asset_response)
+            data = self._download_training_data_from_data_asset_storage()
 
-            # note: read the csv url
-            attachment_url = asset_details['attachments'][0]['handle']['key']
-
-            # note: make the whole url pointing out the csv
-            artifact_content_url = (f"{self._wml_client.data_assets._href_definitions.get_wsd_model_attachment_href()}"
-                                    f"{attachment_url}")
-
-            # note: stream the whole CSV file
-            csv_response = requests.get(artifact_content_url,
-                                        params=self._wml_client._params(),
-                                        headers=self._wml_client._get_headers(),
-                                        stream=True,
-                                        verify=False)
-
-            if csv_response.status_code != 200:
-                raise ApiRequestFailure(u'Failure during {}.'.format("downloading model"), csv_response)
-
-            downloaded_asset = csv_response.content
-
-            # note: read the csv/xlsx file from the memory directly into the pandas DataFrame
-            buffer = io.BytesIO(downloaded_asset)
-            data = try_load_dataset(buffer=buffer,
-                                    sheet_name=self.auto_pipeline_params.get('excel_sheet', 0),
-                                    separator=self.auto_pipeline_params.get('csv_separator', ','))
+        elif self.type == DataConnectionTypes.FS:
+            data = self._download_training_data_from_file_system()
 
         if isinstance(data, DataFrame) and 'Unnamed: 0' in data.columns.tolist():
             data.drop(['Unnamed: 0'], axis=1, inplace=True)
 
+        # note: this code is depreciated, user information isa provided at the beginning.
         if with_holdout_split:
             if not isinstance(data, DataFrame):
                 raise HoldoutSplitNotSupported(
@@ -280,6 +303,12 @@ class DataConnection(BaseDataConnection):
                     reason="SDK currently does not support a local holdout split with xlsx files without sheet_name "
                            "provided.")
 
+            if not self._obm and self.auto_pipeline_params.get('data_join_graph', False):
+                raise HoldoutSplitNotSupported(
+                    None,
+                    reason="Holdout split is not supported for not processed multiple input data. "
+                           "Please use \"get_preprocessed_data().read(with_holdout_split=True)\"")
+
             if self.auto_pipeline_params.get('train_sample_rows_test_size'):
                 pass
 
@@ -307,6 +336,7 @@ class DataConnection(BaseDataConnection):
             data_holdout[self.auto_pipeline_params['prediction_column']] = y_holdout
 
             return data_train, data_holdout
+        # --- end note
 
         return data
 
@@ -447,14 +477,25 @@ class S3Location(BaseLocation):
         if kwargs.get('training_status') is not None:
             self._training_status = kwargs['training_status']
 
+    def _get_file_size(self, cos_resource_client: 'resource') -> 'int':
+        try:
+            size = cos_resource_client.Object(self.bucket, self.path).content_length
+        except ClientError:
+            size = 0
+        return size
+
 
 class FSLocation(BaseLocation):
     """
     Connection class to File Storage in CP4D.
     """
 
-    def __init__(self) -> None:
-        self.path = "{option}/{id}" + f"/assets/auto_ml/autoai_sdk_{uuid.uuid4()}/wml_data"
+    def __init__(self, path: Optional[str] = None) -> None:
+        if path is None:
+            self.path = "{option}/{id}" + f"/assets/auto_ml/auto_ml.{uuid.uuid4()}/wml_data"
+
+        else:
+            self.path = path
 
     @classmethod
     def _set_path(cls, path: str) -> 'FSLocation':
@@ -462,8 +503,57 @@ class FSLocation(BaseLocation):
         location.path = path
         return location
 
+    def _save_file_as_data_asset(self, workspace: 'WorkSpace') -> 'str':
+
+        asset_name = self.path.split('/')[-1]
+        if self.path:
+            data_asset_details = workspace.wml_client.data_assets.create(asset_name, self.path)
+            return workspace.wml_client.data_assets.get_uid(data_asset_details)
+        else:
+            raise MissingValue('path', reason="Incorrect initialization of class FSLocation")
+
+    def _get_file_size(self, workspace: 'WorkSpace') -> 'int':
+        # note if path is not file then returned size is 0
+        try:
+            # note: try to get file size from remote server
+            url = f"{workspace.wml_client.wml_credentials['url']}/v2/asset_files/{self.path.split('/assets/')[-1]}"
+            path_info_response = requests.head(url, headers=workspace.wml_client._get_headers(), params=workspace.wml_client._params(), verify=False)
+            if path_info_response.status_code != 200:
+                raise ApiRequestFailure(u"Failure during getting path details",path_info_response)
+            path_info = path_info_response.headers
+            if 'X-Asset-Files-Type' in path_info and path_info['X-Asset-Files-Type'] == 'file':
+                size = path_info['X-Asset-Files-Size']
+            else:
+                size = 0
+            # -- end note
+        except (ApiRequestFailure, AttributeError):
+            # note try get size of file from local fs
+            size = os.stat(path=self.path).st_size if os.path.isfile(path=self.path) else 0
+            # -- end note
+        return size
+
 
 class DSLocation(BaseLocation):
+
+    def __init__(self, asset_id: str) -> None:
+        self.href = f'/v2/assets/{asset_id}?' + '{option}' + '=' + '{id}'
+
+    @classmethod
+    def _set_path(cls, href: str) -> 'DSLocation':
+        location = cls('.')
+        location.href = href
+        return location
+
+    def _get_file_size(self, workspace: 'WorkSpace', *args) -> 'int':
+        url = workspace.wml_client.wml_credentials['url'] + self.href.format(option=args[0],
+                                                                             id=args[1])
+        asset_info_response = requests.get(url,  headers=workspace.wml_client._get_headers(), verify=False)
+        if asset_info_response.status_code != 200:
+            raise ApiRequestFailure(u"Failure during getting asset details", asset_info_response)
+        return asset_info_response.json()['metadata'].get('size')
+
+
+class CP4DAssetLocation(DSLocation):
     """
     Connection class to data assets in CP4D.
 
@@ -474,10 +564,119 @@ class DSLocation(BaseLocation):
     """
 
     def __init__(self, asset_id: str) -> None:
-        self.href = f'/v2/assets/{asset_id}?' + '{option}' + '=' + '{id}'
+        super().__init__(asset_id)
+
+    def _get_file_size(self, workspace: 'WorkSpace', *args) -> 'int':
+        return super()._get_file_size(workspace, 'project_id', workspace.wml_client.default_project_id)
+
+
+class WMLSAssetLocation(DSLocation):
+    """
+    Connection class to data assets in WML Server.
+
+    Parameters
+    ----------
+    asset_id: str, required
+        Asset ID of the file loaded on space in WML Server.
+    """
+
+    def __init__(self, asset_id: str) -> None:
+        super().__init__(asset_id)
+
+    def _get_file_size(self, workspace: 'WorkSpace', *args) -> 'int':
+        return  super()._get_file_size(workspace, 'space_id', workspace.wml_client.default_space_id)
+
+
+class WSDAssetLocation(BaseLocation):
+    """
+    Connection class to data assets in WS Desktop.
+
+    Parameters
+    ----------
+    asset_id: str, required
+        Asset ID from the project on WS Desktop.
+    """
+
+    def __init__(self, asset_id: str) -> None:
+        self.href = f'/v2/assets/{asset_id}'
+        self._asset_name = None
+        self._asset_id = None
+        self._local_asset_path = None
 
     @classmethod
-    def _set_path(cls, href: str) -> 'DSLocation':
+    def list_assets(cls):
+        project = get_project()
+        return project.get_files()
+
+    def _setup_local_asset_details(self) -> None:
+        # note: set local asset file from asset_id
+        project = get_project()
+        project_id = project.get_metadata()["metadata"]["guid"]
+
+        local_assets = project.get_files()
+
+        # note: reuse local asset_id when object is reused more times
+        if self._asset_id is None:
+            local_asset_id = self.href.split('/')[3].split('?space_id')[0]
+
+        else:
+            local_asset_id = self._asset_id
+        # --- end note
+
+        if local_asset_id not in str(local_assets):
+            raise MissingLocalAsset(local_asset_id, reason="Provided asset_id cannot be found on WS Desktop.")
+
+        else:
+            for asset in local_assets:
+                if asset['asset_id'] == local_asset_id:
+                    asset_name = asset['name']
+                    self._asset_name = asset_name
+                    self._asset_id = local_asset_id
+
+            local_asset_path = f"{os.path.abspath('.')}/{project_id}/assets/data_asset/{asset_name}"
+            self._local_asset_path = local_asset_path
+
+    def _move_asset_to_server(self, workspace: 'WorkSpace') -> None:
+        if not self._local_asset_path or self._asset_name or self._asset_id:
+            self._setup_local_asset_details()
+
+        remote_asset_details = workspace.wml_client.data_assets.create(self._asset_name, self._local_asset_path)
+        self.href = remote_asset_details['metadata']['href']
+
+    @classmethod
+    def _set_path(cls, href: str) -> 'WSDAssetLocation':
         location = cls('.')
         location.href = href
         return location
+
+    def to_dict(self) -> dict:
+        """Return a json dictionary representing this model."""
+        _dict = vars(self).copy()
+        del _dict['_asset_name']
+        del _dict['_asset_id']
+        del _dict['_local_asset_path']
+
+        return _dict
+
+    def _get_file_size(self) -> 'int':
+        if not self._local_asset_path or self._asset_name or self._asset_id:
+            self._setup_local_asset_details()
+        return os.stat(path=self._local_asset_path).st_size if os.path.isfile(path=self._local_asset_path) else 0
+
+
+class DeploymentOutputAssetLocation(BaseLocation):
+    """
+    Connection class to data assets where output of batch deployment will be stored.
+
+    Parameters
+    ----------
+    name: str, required
+        name of .csv file which will be saved as data asset.
+    description: str, optional
+        description of the data asset
+    """
+
+    def __init__(self, name: str, description: str = "") -> None:
+        self.name = name
+        self.description = description
+
