@@ -68,7 +68,7 @@ class LearningSystem(WMLResource):
             :return: learning system configuration details
             :rtype: dict
 
-            **Example**:
+            A way you might use me is
 
             >>> learning_system = client.learning_system.create(model_uid, meta_props=configuration)
         """
@@ -84,16 +84,18 @@ class LearningSystem(WMLResource):
         )
 
         self._handle_response(200, u'creating new model', model_details_response)
-        headers = self._client._get_headers()
+
         if self.ConfigurationMetaNames.SPARK_REFERENCE in meta_props.keys():
             spark_instance = {
                 u'credentials': meta_props[self.ConfigurationMetaNames.SPARK_REFERENCE],
                 u'version': u'2.1'
             }
 
-            headers.update({
+            headers = {
+                u'Content-Type': u'application/json',
+                u'Authorization': u'Bearer ' + self._client.service_instance._get_token(),
                 u'X-Spark-Service-Instance': base64.b64encode(json.dumps(spark_instance).encode('utf-8'))
-            })
+            }
 
             payload = {
                 u'feedback_data_reference': meta_props[self.ConfigurationMetaNames.FEEDBACK_DATA_REFERENCE],
@@ -102,10 +104,10 @@ class LearningSystem(WMLResource):
                 u'auto_redeploy': meta_props[self.ConfigurationMetaNames.AUTO_REDEPLOY]
             }
         elif self.ConfigurationMetaNames.TRAINING_RESULTS_REFERENCE in meta_props.keys():
-            # headers = {
-            #     u'Content-Type': u'application/json',
-            #     u'Authorization': u'Bearer ' + self._client.service_instance._get_token(),
-            # }
+            headers = {
+                u'Content-Type': u'application/json',
+                u'Authorization': u'Bearer ' + self._client.service_instance._get_token(),
+            }
 
             payload = {
                 u'training_results_reference': meta_props[self.ConfigurationMetaNames.TRAINING_RESULTS_REFERENCE],
@@ -137,13 +139,6 @@ class LearningSystem(WMLResource):
 
         :return: metadata of updated learning system
         :rtype: dict
-
-        **Example**
-         >>> metadata = {
-         >>> client.learning_system.ConfigurationMetaNames.NAME:"updated_function"
-         >>> }
-         >>> learning_system_details = client.learning_system.update(model_uid, changes=metadata)
-
         """
         model_uid = str_type_conv(model_uid)
         self._validate_type(model_uid, u'model_uid', STR_TYPE, True)
@@ -168,10 +163,7 @@ class LearningSystem(WMLResource):
             :param model_uid: ID of model for this learning system
             :type model_uid: str
 
-            :returns: learning system details
-            :rtype: dict
-
-            **Example**:
+            A way you might use me is
 
             >>> learning_system_details = client.learning_system.get_details(model_uid)
         """
@@ -191,13 +183,7 @@ class LearningSystem(WMLResource):
         """
             List existing learning systems.
 
-           :returns: None
-           :rtype: None
-
-           .. note::
-               This function only prints the list of existing learning_systems
-
-            **Example**:
+            A way you might use me is
 
             >>> client.learning_system.list()
         """
@@ -220,9 +206,7 @@ class LearningSystem(WMLResource):
             :param asynchronous: if the run should be asynchronous (optional)
             :type asynchronous: bool
 
-            :rtype: dict
-
-            **Example**:
+            A way you might use me is
 
             >>> client.learning_system.run(model_uid)
             >>> client.learning_system.run(model_uid, asynchronous=False)
@@ -297,7 +281,7 @@ class LearningSystem(WMLResource):
             :return: run uid
             :rtype: str
 
-            **Example**:
+            A way you might use me is
 
             >>> run_uid = client.learning_system.get_run_uid(run_details)
         """
@@ -323,7 +307,7 @@ class LearningSystem(WMLResource):
             :return: run href
             :rtype: str
 
-            **Example**:
+            A way you might use me is
 
             >>> run_uid = client.learning_system.get_run_href(run_details)
         """
@@ -346,10 +330,7 @@ class LearningSystem(WMLResource):
             :param model_uid: ID of stored model
             :type model_uid: str
 
-            :return: Learning Iterations
-            :rtype: dict
-
-            **Example**:
+            A way you might use me is
 
             >>> runs_details = client.learning_system.get_runs(model_uid)
         """
@@ -375,7 +356,7 @@ class LearningSystem(WMLResource):
             :param model_uid: ID of stored model
             :type model_uid: str
 
-            **Example**:
+            A way you might use me is
 
             >>> runs_details = client.learning_system.get_metrics(model_uid)
         """
@@ -396,13 +377,7 @@ class LearningSystem(WMLResource):
             :param model_uid: ID of stored model
             :type model_uid: str
 
-           :returns: None
-           :rtype: None
-
-           .. note::
-               This function only prints the evaluation metrics
-
-            **Example**:
+            A way you might use me is
 
             >>> runs_details = client.learning_system.get_metrics(model_uid)
         """
@@ -433,13 +408,7 @@ class LearningSystem(WMLResource):
            :param model_uid: ID of stored model (optional)
            :type model_uid: str
 
-           :returns: None
-           :rtype: None
-
-           .. note::
-               This function only prints the list of learning iterations
-
-           **Example**:
+           A way you might use me is
 
            >>> client.learning_system.list_runs()
         """
@@ -497,7 +466,7 @@ class LearningSystem(WMLResource):
             :param run_uid: ID of learning iteration run
             :type run_uid: str
 
-            **Example**:
+            A way you might use me is
 
             >>> run_details = client.learning_system.get_run_details(run_uid)
         """
@@ -524,7 +493,7 @@ class LearningSystem(WMLResource):
             :param fields: list of fields (optional)
             :type fields: list of strings
 
-            **Example**:
+            A way you might use me is
 
             >>> client.learning_system.send_feedback(model_uid, [["a1", 1, 1.0], ["a2", 2, 3.4]])
             >>> client.learning_system.send_feedback(model_uid, [["a1", 1.0], ["a2", 3.4]], fields=["id", "value"])
