@@ -24,7 +24,7 @@ from watson_machine_learning_client.libs.repo.swagger_client.models.connection_o
 from watson_machine_learning_client.libs.repo.swagger_client.models.ml_assets_create_experiment_input import MlAssetsCreateExperimentInput
 from watson_machine_learning_client.libs.repo.swagger_client.models.array_data_input_repository import  ArrayDataInputRepository
 from watson_machine_learning_client.libs.repo.swagger_client.models.framework_output_repository_runtimes import  FrameworkOutputRepositoryRuntimes
-from  repo.swagger_client.models.framework_output_repository_libraries import  FrameworkOutputRepositoryLibraries
+from watson_machine_learning_client.libs.repo.swagger_client.models.framework_output_repository_libraries import  FrameworkOutputRepositoryLibraries
 from watson_machine_learning_client.libs.repo.mlrepositoryartifact.generic_archive_pipeline_model_artifact import GenericArchivePipelineModelArtifact
 
 logger = logging.getLogger('ExperimentCollection')
@@ -144,13 +144,13 @@ class ExperimentCollection:
             raise ValueError('Invalid type for training_definition_id: {}'.format(training_definition_id.__class__.__name__))
 
         #if not isinstance(ver.encode('ascii'), str):
-        if not isinstance(ver, str) and not isinstance(ver, unicode):
+        if(ver is not None) and not isinstance(ver, str) and not isinstance(ver, unicode):
             raise ValueError('Invalid type for ver: {}'.format(ver.__class__.__name__))
 
         experiment_output = self.repository_api.repository_get_experiment_version(training_definition_id, ver)
 
         if experiment_output is not None:
-            ver_url = experiment_output.entity['training_definition_version']
+            ver_url = experiment_output.metadata['href']
             return ExperimentAdapter(experiment_output, ver_url, self.client).artifact()
             #return self._extract_experiments_from_output(experiment_output)
         else:
@@ -172,18 +172,20 @@ class ExperimentCollection:
         #if artifact_version_url.startswith(self.base_path):
 
         matchedV2 = re.search(
-            '.*/v2/artifacts/pipelines/([A-Za-z0-9\-]+)/versions/([A-Za-z0-9\-]+)', artifact_version_url)
+            '.*/v4/pipelines/([A-Za-z0-9\-]+)$', artifact_version_url)
 
         matchedV3 = re.search(
-            '.*/v3/ml_assets/training_definitions/([A-Za-z0-9\-]+)/versions/([A-Za-z0-9\-]+)', artifact_version_url)
+            '.*/v4/pipelines/([A-Za-z0-9\-]+)?rev=([A-Za-z0-9\-]+)', artifact_version_url)
 
         if matchedV2 is not None:
             training_definition_id = matchedV2.group(1)
-            version_id = matchedV2.group(2)
+            version_id = None
+            print(training_definition_id, version_id)
             return self.version(training_definition_id, version_id)
         elif matchedV3 is not None:
             training_definition_id = matchedV3.group(1)
             version_id = matchedV3.group(2)
+            print (training_definition_id, version_id)
             return self.version(training_definition_id, version_id)
         else:
             raise ValueError('Unexpected artifact version href: {} format'.format(artifact_version_url))
@@ -219,31 +221,46 @@ class ExperimentCollection:
         if not issubclass(type(artifact), PipelineArtifact) and not issubclass(type(artifact),GenericArchivePipelineModelArtifact) :
             raise ValueError('Invalid type for artifact: {}'.format(artifact.__class__.__name__))
 
-        if artifact.meta.prop(MetaNames.TRAINING_DEFINITION_VERSION_URL) is not None:
+        if artifact.meta.prop(MetaNames.PIPELINE_URL) is not None:
             raise ApiException(400, 'Invalid operation: save the same experiment artifact twice')
 
         try:
             training_definition_id = artifact.uid
             if training_definition_id is None:
-                experiment_input = self._prepare_experiment_input(artifact)
-                r = self.repository_api.ml_assets_experiment_creation_with_http_info(experiment_input)
+                lib_name = artifact.meta.prop(MetaNames.NAME)
+                ##create libry
+                name = lib_name
+                description = "custom libraries for scoring"
+                version = "1.0"
+                platform = """{"name":"python", "versions":["3.5"]}"""
+                lib_metadata = {
+                    MetaNames.LIBRARIES.NAME: name,
+                    MetaNames.LIBRARIES.VERSION: version,
+                    MetaNames.LIBRARIES.DESCRIPTION: description,
+                    MetaNames.LIBRARIES.PLATFORM: platform,
+                }
+
+               # experiment_input = self._prepare_experiment_input(artifact)
+
+                r = self.repository_api.v3_libraries_create(library_input)
                 statuscode = r[1]
                 if statuscode is not 201:
-                    logger.info('Error while creating experiment: no location header')
+                    logger.info('Error while creating library: no location header')
                     raise ApiException(404, 'No artifact location')
 
                 experiment_artifact = self._extract_experiments_from_output(r)
                 location = r[2].get('Location')
-                location_match = re.search('.*/v3/ml_assets/training_definitions/([A-Za-z0-9\\-]+)', location)
 
-                if location_match is not None:
-                    training_definition_id = location_match.group(1)
-                else:
-                    logger.info('Error while creating experiment: no location header')
-                    raise ApiException(404, 'No artifact location')
-                artifact_with_guid = artifact._copy(training_definition_id)
-                version_location = experiment_artifact.meta.prop(MetaNames.TRAINING_DEFINITION_VERSION_URL)
-                version_id = experiment_artifact.meta.prop(MetaNames.VERSION)
+                # location_match = re.search('.*/v3/ml_assets/training_definitions/([A-Za-z0-9\\-]+)', location)
+
+                # if location_match is not None:
+                #     training_definition_id = location_match.group(1)
+                # else:
+                #     logger.info('Error while creating experiment: no location header')
+                #     raise ApiException(404, 'No artifact location')
+                # artifact_with_guid = artifact._copy(training_definition_id)
+                # version_location = experiment_artifact.meta.prop(MetaNames.TRAINING_DEFINITION_VERSION_URL)
+                # version_id = experiment_artifact.meta.prop(MetaNames.VERSION)
                 experiment_artifact.pipeline_instance = lambda: artifact.ml_pipeline
 
                 if version_location is not None:
