@@ -152,7 +152,7 @@ class ModelDefinition(WMLResource):
         document = self._generate_model_definition_document(meta_props)
 
         if self._client.WSD:
-            return self._wsd_create_asset("wml_model_definition", document, meta_props, model_definition)
+            return self._wsd_create_asset("wml_model_definition", document, meta_props, model_definition,user_archive_file=True)
 
         model_definition_attachment_def = {
           "asset_type": "wml_model_definition",
@@ -305,22 +305,41 @@ class ModelDefinition(WMLResource):
         ##For CP4D, check if either spce or project ID is set
         self._client._check_if_either_is_set()
         ModelDefinition._validate_type(model_definition_uid, u'model_definition_uid', STR_TYPE, True)
-        attachment_id = self._get_attachment_id(model_definition_uid)
 
-        if not self._ICP:
-            response = requests.get(self._href_definitions.get_attachment_href(model_definition_uid, attachment_id), params=self._client._params(),
-                                    headers=self._client._get_headers())
+
+        if self._client.WSD:
+            import urllib
+            response = requests.get(self._href_definitions.get_data_asset_href(model_definition_uid),
+                                              params=self._client._params(),
+                                              headers=self._client._get_headers())
+
+            model_def_details = self._handle_response(200, u'get model', response)
+            attachment_url = model_def_details['attachments'][0]['object_key']
+            attachment_signed_url = self._href_definitions.get_wsd_model_attachment_href() + \
+                                   urllib.parse.quote('wml_model_definition/' + attachment_url, safe='')
+
         else:
-            response = requests.get(self._href_definitions.get_attachment_href(model_definition_uid, attachment_id), params=self._client._params(),
+            attachment_id = self._get_attachment_id(model_definition_uid)
+            artifact_content_url = self._href_definitions.get_attachment_href(model_definition_uid, attachment_id)
+            if not self._ICP and not self._WSD:
+                response = requests.get(self._href_definitions.get_attachment_href(model_definition_uid, attachment_id), params=self._client._params(),
+                                    headers=self._client._get_headers())
+            else:
+                response = requests.get(artifact_content_url, params=self._client._params(),
                                     headers=self._client._get_headers(), verify=False)
-        if response.status_code == 200:
             attachment_signed_url = response.json()["url"]
-
-            if not self._ICP:
+        if response.status_code == 200:
+            if not self._ICP and not self._client.WSD:
                 att_response = requests.get(self._wml_credentials["url"]+attachment_signed_url)
             else:
-                att_response = requests.get(self._wml_credentials["url"]+attachment_signed_url,
-                                            verify=False)
+                if self._client.WSD:
+                    att_response = requests.get(attachment_signed_url, params= self._client._params(),
+                                                headers=self._client._get_headers(),
+                                                stream=True, verify=False)
+                else:
+                    att_response = requests.get(self._wml_credentials["url"]+attachment_signed_url,
+
+                                                verify=False)
             if att_response.status_code != 200:
                 raise WMLClientError(u'Failure during {}.'.format("downloading model_definition asset"),
                                      att_response)
@@ -363,6 +382,7 @@ class ModelDefinition(WMLResource):
         model_definition_uid = str_type_conv(model_definition_uid)
         ModelDefinition._validate_type(model_definition_uid, u'model_definition_uid', STR_TYPE, True)
         paramvalue = self._client._params()
+
         model_definition_endpoint = self._href_definitions.get_model_definition_assets_href() + "/" + model_definition_uid
         if not self._ICP:
             response_delete = requests.delete(model_definition_endpoint, params=paramvalue, headers=self._client._get_headers())
@@ -407,10 +427,10 @@ class ModelDefinition(WMLResource):
 
         WMLResource._validate_type(response_data,  u'model_definition_response', dict)
         try:
+            href = ""
             if self._client.default_space_id is not None:
                 new_el = {'metadata': {'space_id': response_data['metadata']['space_id'],
                                    'guid': response_data['metadata']['asset_id'],
-                                   'href': response_data['href'],
                                    'asset_type': response_data['metadata']['asset_type'],
                                    'created_at': response_data['metadata']['created_at'],
                                    'last_updated_at': response_data['metadata']['usage']['last_updated_at']
@@ -418,10 +438,12 @@ class ModelDefinition(WMLResource):
                       'entity': response_data['entity']
 
                       }
+                href = "/v2/assets/" + response_data['metadata']['asset_type'] + "/" + response_data['metadata'][
+                    'asset_id'] + "?" + "space_id=" + response_data['metadata']['space_id']
+
             elif self._client.default_project_id is not None:
                 new_el = {'metadata': {'project_id': response_data['metadata']['project_id'],
                                        'guid': response_data['metadata']['asset_id'],
-                                       'href': response_data['href'],
                                        'asset_type': response_data['metadata']['asset_type'],
                                        'created_at': response_data['metadata']['created_at'],
                                        'last_updated_at': response_data['metadata']['usage']['last_updated_at']
@@ -429,9 +451,15 @@ class ModelDefinition(WMLResource):
                           'entity': response_data['entity']
 
                           }
+                href = "/v2/assets/" + response_data['metadata']['asset_type'] + "/" + response_data['metadata'][
+                    'asset_id'] + "?" + "project_id=" + response_data['metadata']['project_id']
+
             if 'revision_id' in response_data['metadata']:
                 new_el['metadata'].update({'rev':response_data['metadata']['revision_id']})
-
+            if 'href' in response_data['metadata']:
+                new_el['metadata'].update({'href': response_data['href']})
+            else:
+                new_el['metadata'].update({'href': href})
             return new_el
         except Exception as e:
             raise WMLClientError("Failed to read Response from down-stream service: " + response_data)
@@ -519,10 +547,14 @@ class ModelDefinition(WMLResource):
                 >>> model_definition_uid = client.model_definitions.get_uid(model_definition_details)
 
         """
-        ModelDefinition._validate_type(model_definition_details, u'model__definition_details', object, True)
-        #ModelDefinition._validate_type_of_details(model_definition_details, MODEL_DEFINITION_DETAILS_TYPE)
+        if 'asset_id' in model_definition_details['metadata']:
+            return WMLResource._get_required_element_from_dict(model_definition_details, u'model_definition_details',
+                                                               [u'metadata', u'asset_id'])
+        else:
+            ModelDefinition._validate_type(model_definition_details, u'model__definition_details', object, True)
+            #ModelDefinition._validate_type_of_details(model_definition_details, MODEL_DEFINITION_DETAILS_TYPE)
 
-        return WMLResource._get_required_element_from_dict(model_definition_details, u'model_definition_details', [u'metadata', u'guid'])
+            return WMLResource._get_required_element_from_dict(model_definition_details, u'model_definition_details', [u'metadata', u'guid'])
 
     def get_href(self, model_definition_details):
         """
@@ -539,13 +571,20 @@ class ModelDefinition(WMLResource):
                 >>> model_definition_uid = client.model_definitions.get_href(model_definition_details)
 
         """
-        ModelDefinition._validate_type(model_definition_details, u'model__definition_details', object, True)
-        # ModelDefinition._validate_type_of_details(model_definition_details, MODEL_DEFINITION_DETAILS_TYPE)
+        if 'asset_id' in model_definition_details['metadata']:
+            return WMLResource._get_required_element_from_dict(model_definition_details, u'model_definition_details', [u'metadata', u'asset_id'])
+        else:
+            ModelDefinition._validate_type(model_definition_details, u'model__definition_details', object, True)
+            # ModelDefinition._validate_type_of_details(model_definition_details, MODEL_DEFINITION_DETAILS_TYPE)
 
-        return WMLResource._get_required_element_from_dict(model_definition_details, u'model_definition_details',
-                                                           [u'metadata', u'href'])
+            return WMLResource._get_required_element_from_dict(model_definition_details, u'model_definition_details',
+                                                               [u'metadata', u'href'])
 
     def create_revision(self, model_definition_uid):
+
+        if self._client.ICP_30 is None:
+            raise WMLClientError(
+                u'Revisions APIs are not supported in this release.')
         url = self._href_definitions.get_model_definition_assets_href() + u'/' + model_definition_uid + u'/revisions'
         paramvalue = self._client._params()
         data = {
@@ -573,6 +612,9 @@ class ModelDefinition(WMLResource):
 
 
     def get_revision_details(self, model_definition_uid, rev_uid):
+        if self._client.ICP_30 is None:
+            raise WMLClientError(
+                u'Revisions APIs are not supported in this release.')
         op_name = 'getting model_definition revision details'
         modeldef_uid = str_type_conv(model_definition_uid)
         ModelDefinition._validate_type(modeldef_uid, u'model_definition_uid', STR_TYPE, True)
@@ -627,6 +669,9 @@ class ModelDefinition(WMLResource):
 
         """
         ##For CP4D, check if either spce or project ID is set
+        if self._client.ICP_30 is None:
+            raise WMLClientError(
+                u'Revisions APIs are not supported in this release.')
         self._client._check_if_either_is_set()
         href = self._href_definitions.get_model_definition_assets_href() + "/" + model_definition_uid +\
                u'/revisions'
