@@ -142,14 +142,15 @@ class Models(WMLResource):
         Store model from object in memory into Watson Machine Learning repository on Cloud
         """
         self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.NAME, STR_TYPE, True)
-        if self._client.ICP_30 is None:
+        if self._client.ICP_30 is None and self._client.WSD_20 is None:
             if self.ConfigurationMetaNames.SOFTWARE_SPEC_UID in meta_props and \
                self.ConfigurationMetaNames.RUNTIME_UID not in meta_props:
                 raise WMLClientError("Invalid input. It is mandatory to provide RUNTIME_UID in meta_props.")
         else:
             if self.ConfigurationMetaNames.SOFTWARE_SPEC_UID not in meta_props and \
                self.ConfigurationMetaNames.RUNTIME_UID not in meta_props:
-                raise WMLClientError("Invalid input. It is mandatory to provide RUNTIME_UID or SOFTWARE_SPEC_UID in meta_props.")
+                  raise WMLClientError("Invalid input. It is mandatory to provide RUNTIME_UID or "
+                                       "SOFTWARE_SPEC_UID in meta_props. RUNTIME_UID is deprecated")
 
         # self._validate_type(project_id, 'project_id',STR_TYPE, True)
         try:
@@ -245,10 +246,15 @@ class Models(WMLResource):
                     if self._client.ICP_30:
                         self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE, False)
                         meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID] = {"id": meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}
+                    elif self._client.WSD_20:
+                        self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE,
+                                                 False)
+                        meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID] = {
+                            "base_id": meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}
                     else:
                         meta_props.pop(self.ConfigurationMetaNames.SOFTWARE_SPEC_UID)
 
-                if self._client.ICP_30:
+                if self._client.ICP_30 or self._client.WSD_20:
                     if self.ConfigurationMetaNames.MODEL_DEFINITION_UID in meta_props:
                         self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.MODEL_DEFINITION_UID, STR_TYPE, False)
                         meta_props[self._client.repository.ModelMetaNames.MODEL_DEFINITION_UID] = {"id":meta_props[self._client.repository.ModelMetaNames.MODEL_DEFINITION_UID]}
@@ -282,6 +288,11 @@ class Models(WMLResource):
                         self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE, False)
                         meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID] = \
                             {"id": meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}
+                    elif self._client.WSD_20:
+                        self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE,
+                                                 False)
+                        meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID] = \
+                            {"base_id": meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}
                     else:
                         meta_props.pop(self.ConfigurationMetaNames.SOFTWARE_SPEC_UID)
 
@@ -292,7 +303,7 @@ class Models(WMLResource):
                     else:
                         meta_props[self._client.repository.ModelMetaNames.PIPELINE_UID] = {"href": API_VERSION + PIPELINES + "/" + meta_props[self._client.repository.ModelMetaNames.PIPELINE_UID]}
 
-                if self._client.ICP_30:
+                if self._client.ICP_30 or self._client.WSD_20:
                     if self.ConfigurationMetaNames.MODEL_DEFINITION_UID in meta_props:
                         self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.MODEL_DEFINITION_UID, STR_TYPE,
                                                  False)
@@ -362,6 +373,10 @@ class Models(WMLResource):
                 self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE, False)
                 model_meta.update({self.ConfigurationMetaNames.SOFTWARE_SPEC_UID: {
                     "id": meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}})
+            elif self._client.WSD_20:
+                self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE, False)
+                model_meta.update({self.ConfigurationMetaNames.SOFTWARE_SPEC_UID: {
+                    "base_id": meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}})
             else:
                 model_meta.pop(self.ConfigurationMetaNames.SOFTWARE_SPEC_UID)
 
@@ -396,7 +411,7 @@ class Models(WMLResource):
         output_schema = []
         if self.ConfigurationMetaNames.INPUT_DATA_SCHEMA in meta_props and \
            meta_props[self.ConfigurationMetaNames.INPUT_DATA_SCHEMA] is not None:
-            if self._client.ICP_30:
+            if self._client.ICP_30 or self._client.WSD_20:
                 if isinstance(meta_props[self.ConfigurationMetaNames.INPUT_DATA_SCHEMA], dict):
                     input_schema = [meta_props[self.ConfigurationMetaNames.INPUT_DATA_SCHEMA]]
                 else:
@@ -407,7 +422,8 @@ class Models(WMLResource):
                 input_schema = meta_props[self.ConfigurationMetaNames.INPUT_DATA_SCHEMA]
             model_meta.pop(self.ConfigurationMetaNames.INPUT_DATA_SCHEMA)
         if self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMA in meta_props:
-            if str(meta_props[self.ConfigurationMetaNames.TYPE]).startswith('do-') and self._client.ICP_30:
+            if str(meta_props[self.ConfigurationMetaNames.TYPE]).startswith('do-') and \
+                    (self._client.ICP_30 or self._client.WSD_20):
                 try:
                     self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMA, dict, False)
                     output_schema = [meta_props[self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMA]]
@@ -547,7 +563,13 @@ class Models(WMLResource):
         )
         self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.NAME, STR_TYPE, True)
 
-        x = re.findall(r"[0-9A-Za-z-]+-[0-9A-Za-z-]+", model_path)
+        # note: remove pipeline-model.json part from the string to allow correct regexp
+        if model_path.endswith('pipeline-model.json'):
+            x = re.findall(r"[0-9A-Za-z-]+-[0-9A-Za-z-]+", '/'.join(model_path.split('/')[:-1]))
+            # --- end note
+        else:
+            x = re.findall(r"[0-9A-Za-z-]+-[0-9A-Za-z-]+", model_path)
+
         model_uid = x[-1] if x else ''
         details = self._client.training.get_details(model_uid)
 
@@ -560,6 +582,10 @@ class Models(WMLResource):
                 self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE, False)
                 model_meta.update({self.ConfigurationMetaNames.SOFTWARE_SPEC_UID: {
                     "id": meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}})
+            elif self._client.WSD_20:
+                self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE, False)
+                model_meta.update({self.ConfigurationMetaNames.SOFTWARE_SPEC_UID: {
+                    "base_id": meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}})
             else:
                 model_meta.pop(self.ConfigurationMetaNames.SOFTWARE_SPEC_UID)
 
@@ -612,7 +638,7 @@ class Models(WMLResource):
         output_schema = []
         if self.ConfigurationMetaNames.INPUT_DATA_SCHEMA in meta_props and \
                 meta_props[self.ConfigurationMetaNames.INPUT_DATA_SCHEMA] is not None:
-            if self._client.ICP_30:
+            if self._client.ICP_30 or self._client.WSD_20:
                 if isinstance(meta_props[self.ConfigurationMetaNames.INPUT_DATA_SCHEMA], dict):
                     input_schema = [meta_props[self.ConfigurationMetaNames.INPUT_DATA_SCHEMA]]
                 else:
@@ -625,7 +651,8 @@ class Models(WMLResource):
 
         if self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMA in meta_props and \
                 meta_props[self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMA] is not None:
-            if str(meta_props[self.ConfigurationMetaNames.TYPE]).startswith('do-') and self._client.ICP_30:
+            if str(meta_props[self.ConfigurationMetaNames.TYPE]).startswith('do-') and \
+                    (self._client.ICP_30 or self._client.WSD_20):
                 try:
                     self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMA, dict, False)
                     output_schema = [meta_props[self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMA]]
@@ -664,14 +691,15 @@ class Models(WMLResource):
         """
         Store saved model into Watson Machine Learning repository on IBM Cloud
         """
-        if not self._client.ICP_30:
+        if not self._client.ICP_30 and not self._client.WSD_20:
             if self.ConfigurationMetaNames.SOFTWARE_SPEC_UID in meta_props and \
               self.ConfigurationMetaNames.RUNTIME_UID not in meta_props:
                 raise WMLClientError("Invalid input. It is mandatory to provide RUNTIME_UID in metaprop.")
         else:
             if self.ConfigurationMetaNames.SOFTWARE_SPEC_UID not in meta_props and \
                self.ConfigurationMetaNames.RUNTIME_UID not in meta_props:
-                raise WMLClientError("Invalid input. It is mandatory to provide RUNTIME_UID in metaprop.")
+                raise WMLClientError("Invalid input. It is mandatory to provide RUNTIME_UID or "
+                                     "SOFTWARE_SPEC_UID in meta_props. RUNTIME_UID is deprecated")
 
         if(ver==True):
             #check if artifactid is passed
@@ -749,8 +777,14 @@ class Models(WMLResource):
                         self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE, False)
                         meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID] = {
                             "id": meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}
+                    elif self._client.WSD_20:
+                        self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE,
+                                                 False)
+                        meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID] = {
+                            "base_id": meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}
                     else:
                         meta_props.pop(self.ConfigurationMetaNames.SOFTWARE_SPEC_UID)
+
                 if self.ConfigurationMetaNames.TRAINING_LIB_UID in meta_props:
                     self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.TRAINING_LIB_UID, STR_TYPE, False)
                     meta_props[self._client.repository.ModelMetaNames.TRAINING_LIB_UID] = {"href": API_VERSION + LIBRARIES + "/" + meta_props[self._client.repository.ModelMetaNames.TRAINING_LIB_UID]}
@@ -859,10 +893,13 @@ class Models(WMLResource):
                 "href": API_VERSION + RUNTIMES + "/" + meta_props[self._client.repository.ModelMetaNames.RUNTIME_UID]}})
         if self.ConfigurationMetaNames.SOFTWARE_SPEC_UID in meta_props:
             if self._client.ICP_30:
-                if not self._client.WSD:
-                    self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE, False)
+                self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE, False)
                 payload.update({self.ConfigurationMetaNames.SOFTWARE_SPEC_UID: {
                     "id": meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}})
+            if self._client.WSD_20:
+                self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.SOFTWARE_SPEC_UID, STR_TYPE, False)
+                payload.update({self.ConfigurationMetaNames.SOFTWARE_SPEC_UID: {
+                    "base_id": meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}})
 
         if self.ConfigurationMetaNames.PIPELINE_UID in meta_props:
             self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.PIPELINE_UID, STR_TYPE, False)
@@ -874,7 +911,7 @@ class Models(WMLResource):
                     "href": API_VERSION + PIPELINES + "/" + meta_props[
                         self._client.repository.ModelMetaNames.PIPELINE_UID]}})
 
-        if self._client.ICP_30:
+        if self._client.ICP_30 or self._client.WSD_20:
             if self.ConfigurationMetaNames.MODEL_DEFINITION_UID in meta_props:
                 self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.MODEL_DEFINITION_UID, STR_TYPE, False)
                 payload.update({self.ConfigurationMetaNames.MODEL_DEFINITION_UID: {
@@ -922,7 +959,7 @@ class Models(WMLResource):
         output_schema = []
         if self.ConfigurationMetaNames.INPUT_DATA_SCHEMA in meta_props and \
                 meta_props[self.ConfigurationMetaNames.INPUT_DATA_SCHEMA] is not None:
-            if self._client.ICP_30:
+            if self._client.ICP_30 or self._client.WSD_20:
                 if isinstance(meta_props[self.ConfigurationMetaNames.INPUT_DATA_SCHEMA], dict):
                     input_schema = [meta_props[self.ConfigurationMetaNames.INPUT_DATA_SCHEMA]]
                 else:
@@ -934,7 +971,8 @@ class Models(WMLResource):
 
         if self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMA in meta_props and \
                 meta_props[self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMA] is not None:
-            if str(meta_props[self.ConfigurationMetaNames.TYPE]).startswith('do-') and self._client.ICP_30:
+            if str(meta_props[self.ConfigurationMetaNames.TYPE]).startswith('do-') and \
+                    (self._client.ICP_30 or self._client.WSD_20):
                 try:
                     self._validate_meta_prop(meta_props, self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMA, dict, False)
                     output_schema = [meta_props[self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMA]]
@@ -1150,8 +1188,9 @@ class Models(WMLResource):
                 saved_model = self._publish_from_object(model=model, meta_props=meta_props, training_data=training_data, training_target=training_target, pipeline=pipeline, feature_names=feature_names, label_column_names=label_column_names)
         else:
             # print("model is a string type")
-            if (model.endswith('.pickle') and os.path.sep in model):
+            if ((model.endswith('.pickle') or model.endswith('pipeline-model.json')) and os.path.sep in model):
                 # AUTO AI Trained model
+                # pipeline-model.json is needed for OBM + KB
                 saved_model = self._store_autoAI_model(model_path=model, meta_props=meta_props, training_data=training_data, training_target=training_target, version=version, artifactId=artifactid, subtrainingId=subtrainingId)
 
             elif (os.path.sep in model) or os.path.isfile(model) or os.path.isdir(model):
@@ -1560,7 +1599,7 @@ class Models(WMLResource):
 
         if self._client.ICP_30 is None:
             raise WMLClientError(
-                u'Revision support is not there in this WML server. It is supported only from 3.1.0 onwards.')
+                u'Revisions APIs are not supported in this release.')
         else:
             url = self._href_definitions.get_published_models_href()
             return self._create_revision_artifact(url, model_uid, 'models')
@@ -1589,7 +1628,7 @@ class Models(WMLResource):
 
         if self._client.ICP_30 is None:
             raise WMLClientError(
-                u'Revision support is not there in this WML server. It is supported only from 3.1.0 onwards.')
+                u'Revision APIs are not supported in this release.')
         else:
             url = self._href_definitions.get_published_models_href() + "/" + model_uid
             model_resources = self._get_artifact_details(url, "revisions", limit, 'model revisions')[u'resources']
@@ -1625,7 +1664,7 @@ class Models(WMLResource):
 
         if not self._client.ICP_30:
             raise WMLClientError(
-                'Not supported. Revisions APIs are supported only for WML Cloud Pack for Data 3.0 and above.')
+                'Revision APIs are not supported in this release.')
         else:
             url = self._href_definitions.get_published_models_href() + "/" + model_uid
             return self._get_with_or_without_limit(url, limit=None, op_name="model",
