@@ -883,7 +883,11 @@ class Runtimes(WMLResource):
                 print(u'Successfully downloaded runtime configuration file: ' + str(filename))
                 return os.getcwd() + "/"+filename
         else:
-            raise WMLClientError(u'Unable to download configuration content: ' + response_get.text)
+            if response_get.status_code == 404 and "content_does_not_exist" in str(response_get.text):
+                raise WMLClientError(u'Unable to download configuration. download configruation can be invoked '
+                                     u'only when CONFIGURATION_FILEPATH meta prop is set during store. ')
+            else:
+                raise WMLClientError(u'Unable to download configuration content: ' + response_get.text)
 
     def download_library(self, library_uid, filename=None):
         """
