@@ -154,7 +154,7 @@ class Shiny(WMLResource):
             meta_props = {
                     "name": name
             }
-            details = Shiny._wsd_create_asset(self, "shiny_asset", asset_meta, meta_props, file_path)
+            details = Shiny._wsd_create_asset(self, "shiny_asset", asset_meta, meta_props, file_path, user_archive_file=True)
             return self._get_required_element_from_response(details)
         else:
 
@@ -771,12 +771,6 @@ class Shiny(WMLResource):
                                        headers=self._client._get_headers(), verify=False)
 
 
-        # if response.status_code == 200:
-        #     return self._get_required_element_from_response(response.json())
-        # else:
-        #     return self._handle_response(204, u'delete assets', response)
-
-
     # @docstring_parameter({'str_type': STR_TYPE_NAME})
     # def get_href(self, asset_uid):
     #     """
@@ -862,11 +856,9 @@ class Shiny(WMLResource):
                                 'href': href,
                                 'name': response_data[u'metadata'][u'name'],
                                 'asset_type': response_data['metadata']['asset_type'],
-                                'created_at': response_data['metadata']['created_at'],
-                                'last_updated_at': response_data['metadata']['usage']['last_updated_at']
+                                'created_at': response_data['metadata']['created_at']
                     }
-
-                    if self._client.ICP_30 is not None:
+                    if self._client.WSD_20 is not None:
                         if "revision_id" in response_data[u'metadata']:
                             revision_id = response_data[u'metadata'][u'revision_id']
                             metadata.update({'revision_id': response_data[u'metadata'][u'revision_id']})
@@ -881,6 +873,13 @@ class Shiny(WMLResource):
                     new_el = {'metadata': metadata,
                               'entity': response_data['entity']
                               }
+                    if 'usage' in response_data['metadata']:
+                        new_el['metadata'].update(
+                            {'last_updated_at': response_data['metadata']['usage']['last_updated_at']})
+                    else:
+                        new_el['metadata'].update(
+                            {'last_updated_at': response_data['metadata']['last_updated_at']})
+
                 else:
                     metadata = {'project_id': response_data['metadata']['project_id'],
                                 'guid': response_data['metadata']['asset_id'],
