@@ -25,10 +25,43 @@ class SwSpec(WMLResource):
     """
     ConfigurationMetaNames = SwSpecMetaNames()
     """MetaNames for Software Specification creation."""
-
     def __init__(self, client):
         WMLResource.__init__(self, __name__, client)
         self._ICP = client.ICP
+        self.software_spec_list = None
+        if self._client.WSD_20:
+            self.software_spec_list = {
+                                "default_py3.6": "0062b8c9-8b7d-44a0-a9b9-46c416adcbd9",
+                                "scikit-learn_0.20-py3.6": "09c5a1d0-9c1e-4473-a344-eb7b665ff687",
+                                "ai-function_0.1-py3.6": "0cdb0f1e-5376-4f4d-92dd-da3b69aa9bda",
+                                "shiny-r3.6": "0e6e79df-875e-4f24-8ae9-62dcc2148306",
+                                "pytorch_1.1-py3.6": "10ac12d6-6b30-4ccd-8392-3e922c096a92" ,
+                                "scikit-learn_0.22-py3.6": "154010fa-5b3b-4ac1-82af-4d5ee5abbc85",
+                                "default_r3.6":  "1b70aec3-ab34-4b87-8aa0-a4a3c8296a36",
+                                "tensorflow_1.15-py3.6":  "2b73a275-7cbf-420b-a912-eae7f436e0bc",
+                                "pytorch_1.2-py3.6":  "2c8ef57d-2687-4b7d-acce-01f94976dac1",
+                                "spark-mllib_2.3":  "2e51f700-bca0-4b0d-88dc-5c6791338875",
+                                "pytorch-onnx_1.1-py3.6-edt": "32983cea-3f32-4400-8965-dde874a8d67e",
+                                "spark-mllib_2.4": "390d21f8-e58b-4fac-9c55-d7ceda621326",
+                                "xgboost_0.82-py3.6":  "39e31acd-5f30-41dc-ae44-60233c80306e",
+                                "pytorch-onnx_1.2-py3.6-edt": "40589d0e-7019-4e28-8daa-fb03b6f4fe12",
+                                "spark-mllib_2.4-r_3.6": "49403dff-92e9-4c87-a3d7-a42d0021c095",
+                                "xgboost_0.90-py3.6": "4ff8d6c2-1343-4c18-85e1-689c965304d3",
+                                "pytorch-onnx_1.1-py3.6":  "50f95b2a-bc16-43bb-bc94-b0bed208c60b",
+                                "spark-mllib_2.4-scala_2.11": "55a70f99-7320-4be5-9fb9-9edb5a443af5",
+                                "spss-modeler_18.1":  "5c3cad7e-507f-4b2a-a9a3-ab53a21dee8b",
+                                "spark-mllib_2.3-r_3.6":  "6586b9e3-ccd6-4f92-900f-0f8cb2bd6f0c",
+                                "spss-modeler_18.2":  "687eddc9-028a-4117-b9dd-e57b36f1efa5",
+                                "pytorch-onnx_1.2-py3.6":  "692a6a4d-2c4d-45ff-a1ed-b167ee55469a",
+                                "do_12.9":  "75a3a4b0-6aa0-41b3-a618-48b1f56332a6",
+                                "spark-mllib_2.3-scala_2.11": "7963efe5-bbec-417e-92cf-0574e21b4e8d",
+                                "caffe_1.0-py3.6":  "7bb3dbe2-da6e-4145-918d-b6d84aa93b6b",
+                                "cuda-py3.6":  "82c79ece-4d12-40e6-8787-a7b9e0f62770",
+                                "hybrid_0.1":  "8c1a58c6-62b5-4dc4-987a-df751c2756b6",
+                                "caffe-ibm_1.0-py3.6":  "8d863266-7927-4d1e-97d7-56a7f4c0a19b",
+                                "spss-modeler_17.1":  "902d0051-84bd-4af6-ab6b-8f6aa6fdeabb",
+                                "do_12.10":  "9100fd72-8159-4eb9-8a0b-a87e12eefa36",
+                                "hybrid_0.2":  "9b3f9040-9cee-4ead-8d7a-780600f542f7"}
 
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def get_details(self,sw_spec_uid=None):
@@ -52,6 +85,9 @@ class SwSpec(WMLResource):
              >>> sw_spec_details = client.software_specifications.get_details(sw_spec_uid)
 
         """
+        if self._client.WSD_20:
+            raise WMLClientError(u'get_details API is not supported in Watson Studio Desktop.')
+
         SwSpec._validate_type(sw_spec_uid, u'sw_spec_uid', STR_TYPE, True)
 
 
@@ -97,6 +133,8 @@ class SwSpec(WMLResource):
                  >>> }
 
         """
+        if self._client.WSD_20:
+            raise WMLClientError(u'store() API is not supported in Watson Studio Desktop.')
 
         SwSpec._validate_type(meta_props, u'meta_props', dict, True)
         sw_spec_meta = self.ConfigurationMetaNames._generate_resource_metadata(
@@ -132,22 +170,26 @@ class SwSpec(WMLResource):
             >>> client.software_specifications.list()
         """
 
-        # Todo provide api to return
-        href = self._href_definitions.get_sw_specs_href()
+        if not self._client.WSD_20:
+            # Todo provide api to return
+            href = self._href_definitions.get_sw_specs_href()
 
 
-        if not self._ICP:
-            response = requests.get(href, params=self._client._params(), headers=self._client._get_headers())
+            if not self._ICP:
+                response = requests.get(href, params=self._client._params(), headers=self._client._get_headers())
+            else:
+                response = requests.get(href, params=self._client._params(), headers=self._client._get_headers(), verify=False)
+            self._handle_response(200, u'list sw_specs', response)
+            asset_details = self._handle_response(200, u'list assets', response)["resources"]
+            sw_spec_values = [
+                (m[u'metadata'][u'name'], m[u'metadata'][u'asset_id'],m[u'entity'][u'software_specification'][u'type']) for
+                m in asset_details]
+
+            self._list(sw_spec_values, [u'NAME', u'ASSET_ID', u'TYPE'], None, _DEFAULT_LIST_LENGTH)
         else:
-            response = requests.get(href, params=self._client._params(), headers=self._client._get_headers(), verify=False)
-        self._handle_response(200, u'list sw_specs', response)
-        asset_details = self._handle_response(200, u'list assets', response)["resources"]
-        sw_spec_values = [
-            (m[u'metadata'][u'name'], m[u'metadata'][u'asset_id'],m[u'entity'][u'software_specification'][u'type']) for
-            m in asset_details]
-
-        self._list(sw_spec_values, [u'NAME', u'ASSET_ID', u'TYPE'], None, _DEFAULT_LIST_LENGTH)
-
+            from tabulate import tabulate
+            header = [u'NAME', u'ASSET_ID', u'TYPE']
+            print(tabulate(self.software_spec_list.items(), headers=header))
 
     @staticmethod
     @docstring_parameter({'str_type': STR_TYPE_NAME})
@@ -210,25 +252,27 @@ class SwSpec(WMLResource):
         """
 
         SwSpec._validate_type(sw_spec_name, u'sw_spec_uid', STR_TYPE, True)
-        parameters = self._client._params()
-        parameters.update(name=sw_spec_name)
-        if not self._ICP:
-            response = requests.get(self._href_definitions.get_sw_specs_href(),
-                                    params=parameters,
-                                    headers=self._client._get_headers())
-        else:
-
-            response = requests.get(self._href_definitions.get_sw_specs_href(),
-                                    params=parameters,
-                                    headers=self._client._get_headers(), verify=False)
-        if response.status_code == 200:
-            total_values = self._handle_response(200, u'list assets', response)["total_results"]
-            if total_values != 0:
-                sw_spec_details = self._handle_response(200, u'list assets', response)["resources"]
-                return sw_spec_details[0][u'metadata'][u'asset_id']
+        if not self._client.WSD_20:
+            parameters = self._client._params()
+            parameters.update(name=sw_spec_name)
+            if not self._ICP:
+                response = requests.get(self._href_definitions.get_sw_specs_href(),
+                                        params=parameters,
+                                        headers=self._client._get_headers())
             else:
-                return "Not Found"
 
+                response = requests.get(self._href_definitions.get_sw_specs_href(),
+                                        params=parameters,
+                                        headers=self._client._get_headers(), verify=False)
+            if response.status_code == 200:
+                total_values = self._handle_response(200, u'list assets', response)["total_results"]
+                if total_values != 0:
+                    sw_spec_details = self._handle_response(200, u'list assets', response)["resources"]
+                    return sw_spec_details[0][u'metadata'][u'asset_id']
+                else:
+                    return "Not Found"
+        else:
+            return self.software_spec_list.get(sw_spec_name)
 
 
     @staticmethod
@@ -283,6 +327,9 @@ class SwSpec(WMLResource):
              >>> client.software_specifications.delete(sw_spec_uid)
 
         """
+        if self._client.WSD_20:
+            raise WMLClientError(u'delete API is not supported in Watson Studio Desktop.')
+
         SwSpec._validate_type(sw_spec_uid, u'sw_spec_uid', STR_TYPE, True)
 
         if not self._ICP:
@@ -316,6 +363,9 @@ class SwSpec(WMLResource):
                 >>> client.software_specifications.add_package_extension(sw_spec_uid, pkg_extn_id)
 
         """
+        if self._client.WSD_20:
+            raise WMLClientError(u'package extension APIs are not supported in Watson Studio Desktop.')
+
         ##For CP4D, check if either spce or project ID is set
         self._client._check_if_either_is_set()
 
@@ -356,6 +406,8 @@ class SwSpec(WMLResource):
                 >>>> client.software_specifications.delete_package_extension(sw_spec_uid, pkg_extn_id)
 
         """
+        if self._client.WSD_20:
+            raise WMLClientError(u'package extension APIs are not supported in Watson Studio Desktop.')
 
         ##For CP4D, check if either spce or project ID is set
         self._client._check_if_either_is_set()
