@@ -32,9 +32,9 @@ class TestAutoAIRemote(unittest.TestCase):
     service: 'WebService' = None
 
     data_dir = './autoai/data/xlsx'
-    binary_file_name = 'credit_risk_two_sheets.xlsx'
+    binary_file_name = 'insurance_credit_risk__two_sheets.xlsx' # 'credit_risk_two_sheets.xlsx'
     multiclass_file_name = 'iris_dataset.xlsx'
-    regression_file_name = 'insurance_two_sheets.xlsx'
+    regression_file_name = 'credit_risk_insurance__two_sheets.xlsx'# 'insurance_two_sheets.xlsx'
 
     bin_target_column = 'Risk'
     multi_target_column = 'species'
@@ -166,7 +166,7 @@ class TestAutoAIRemote(unittest.TestCase):
             prediction_column=self.bin_target_column,
             positive_label='Risk',
             scoring=PositiveLabelClass.PRECISION_SCORE_WEIGHTED,
-            excel_sheet='credit_risk_training_light'
+            excel_sheet=1 #'credit_risk_training_light'
         )
 
         self.assertIsInstance(self.remote_auto_pipelines, RemoteAutoPipelines,
@@ -188,6 +188,8 @@ class TestAutoAIRemote(unittest.TestCase):
         self.assertIsNotNone(self.data_connection.auto_pipeline_params,
                              msg='DataConnection auto_pipeline_params was not updated.')
 
+    def test_05a__binary__get_train_holdout_data(self):
+
         TestAutoAIRemote.train_data, TestAutoAIRemote.holdout_data = self.remote_auto_pipelines.get_data_connections()[0].read(with_holdout_split=True)
 
         print("train data sample:")
@@ -336,7 +338,8 @@ class TestAutoAIRemote(unittest.TestCase):
             name=self.binary_file_name.split('.')[0],
             prediction_type=AutoAI.PredictionType.REGRESSION,
             prediction_column=self.regression_target_column,
-            scoring=AutoAI.Metrics.ROOT_MEAN_SQUARED_ERROR
+            scoring=AutoAI.Metrics.ROOT_MEAN_SQUARED_ERROR,
+            excel_sheet='insurance'
         )
 
         self.assertIsInstance(self.remote_auto_pipelines, RemoteAutoPipelines,
