@@ -9,25 +9,41 @@ from os import environ
 
 from watson_machine_learning_client import WatsonMachineLearningAPIClient
 from watson_machine_learning_client.experiment import AutoAI
-from watson_machine_learning_client.helpers.connections import S3Connection, S3Location, DataConnection, DSLocation
+from watson_machine_learning_client.helpers.connections import S3Connection, S3Location, DataConnection, DSLocation, FSLocation
 from watson_machine_learning_client.experiment.autoai.optimizers import RemoteAutoPipelines
-from watson_machine_learning_client.deployment import WebService
+from watson_machine_learning_client.deployment import WebService, Batch
+from watson_machine_learning_client.workspace import WorkSpace
+from watson_machine_learning_client.utils.autoai.enums import TShirtSize
+from watson_machine_learning_client.utils.autoai.errors import AutoAIComputeError
 
-from watson_machine_learning_client.tests.utils import get_wml_credentials, get_cos_credentials, is_cp4d, bucket_exists, bucket_name_gen
-from watson_machine_learning_client.tests.utils.cleanup import delete_model_deployment, bucket_cleanup
+from watson_machine_learning_client.tests.utils import get_wml_credentials, get_cos_credentials, is_cp4d, bucket_exists,\
+    bucket_name_gen, get_space_id
+from watson_machine_learning_client.tests.utils.cleanup import delete_model_deployment, bucket_cleanup, space_cleanup
 
 
 @unittest.skipIf(is_cp4d(), "Not supported on CP4D")
 class TestAutoAIRemote(unittest.TestCase):
+    """
+    The test covers:
+    - COS set-up (if run on Cloud): checking if bucket exists for the cos instance, if not new bucket is create
+    - Saving data `datasets_different_delimeter/iris_dataset_different_delimeter.csv` to COS/data assets
+    - multiclass experiment for datasets with different delimeter dataset and `;` separator
+    - downloading all generated pipelines to lale pipeline
+    - list historical runs
+    - get one historical run and download its pipeline
+    - predict on historical run
+    - deployment by `pipeline name`
+    - deployment deletion
+    """
     wml_client: 'WatsonMachineLearningAPIClient' = None
     experiment: 'AutoAI' = None
     remote_auto_pipelines: 'RemoteAutoPipelines' = None
     service: 'WebService' = None
+    service_batch: 'Batch' = None
     wml_credentials = None
     cos_credentials = None
 
     train_data = None
-    holdout_data = None
 
     pipeline_opt: 'RemoteAutoPipelines' = None
     historical_opt: 'RemoteAutoPipelines' = None
@@ -45,20 +61,21 @@ class TestAutoAIRemote(unittest.TestCase):
     data_connection = None
     results_connection = None
 
-    data_location = './autoai/data/iris_dataset.csv'
-
+    data_location = './autoai/data/datasets_different_delimeter/iris_dataset_different_delimeter.csv'
+    custom_separator = ';'
     # CLOUD CONNECTION DETAILS:
 
     cos_endpoint = "https://s3.us-south.cloud-object-storage.appdomain.cloud"
 
-    if "BUCKET_NAME" in environ:
-        bucket_name = environ['BUCKET_NAME']
-    else:
-        bucket_name = "wml-autoaitests-qa"
-    data_cos_path = 'data/iris_dataset.csv'
+    bucket_name = environ.get('BUCKET_NAME', "wml-autoaitests-qa")
+    # pod_version = environ.get('KB_VERSION', None)
+
+    data_cos_path = 'data/iris_dataset_different_delimeter.csv'
 
     results_cos_path = 'results_wml_autoai'
 
+    space_name = 'tests_sdk_space'
+
     OPTIMIZER_NAME = 'Iris wml_autoai multiclass test'
     DEPLOYMENT_NAME = "Iris AutoAI Deployment tests"
 
@@ -72,20 +89,31 @@ class TestAutoAIRemote(unittest.TestCase):
         """
         Load WML credentials from config.ini file based on ENV variable.
         """
-        cls.data = pd.read_csv(cls.data_location)
-        cls.X = cls.data.drop(['species'], axis=1)
-        cls.y = cls.data['species']
 
         cls.wml_credentials = get_wml_credentials()
 
-        if not is_cp4d():
+        cls.wml_client = WatsonMachineLearningAPIClient(wml_credentials=cls.wml_credentials.copy())
+        if not cls.wml_client.ICP:
             cls.cos_credentials = get_cos_credentials()
             if 'endpoint_url' in cls.cos_credentials:
                 cls.cos_endpoint = cls.cos_credentials['endpoint_url']
+        else:
+            # note: set space for WMLS
+            space_cleanup(cls.wml_client, get_space_id(cls.wml_client, cls.space_name), days_old=2)
+            cls.space_id = get_space_id(cls.wml_client, cls.space_name)
+            cls.project_id = None
+            # -- end note
+
+            # note: set comma-separated csv as data for WMLS (custom separator and xlsx files are not supported)
+            cls.custom_separator = ','
+            cls.data_location = './autoai/data/iris_dataset.csv'
+            # -- end note
 
         cls.wml_client = WatsonMachineLearningAPIClient(wml_credentials=cls.wml_credentials.copy())
 
     def test_00_prepare_COS_instance(self):
+        if self.wml_client.ICP or self.wml_client.WSD:
+            self.skipTest("Prepare COS is available only for Cloud")
         import ibm_boto3
         cos_resource = ibm_boto3.resource(
             service_name="s3",
@@ -111,7 +139,7 @@ class TestAutoAIRemote(unittest.TestCase):
         print(f"Using COS bucket: {TestAutoAIRemote.bucket_name}")
 
     def test_01_initialize_AutoAI_experiment__pass_credentials__object_initialized(self):
-        if is_cp4d():
+        if self.wml_client.ICP:
             TestAutoAIRemote.experiment = AutoAI(wml_credentials=self.wml_credentials.copy(),
                                                  project_id=self.project_id,
                                                  space_id=self.space_id)
@@ -121,15 +149,9 @@ class TestAutoAIRemote(unittest.TestCase):
         self.assertIsInstance(self.experiment, AutoAI, msg="Experiment is not of type AutoAI.")
 
     def test_02_save_remote_data_and_DataConnection_setup(self):
-        if is_cp4d():
-            self.wml_client.set.default_project(self.project_id)
-            asset_details = self.wml_client.data_assets.create(
-                name=self.data_location.split('/')[-1],
-                file_path=self.data_location)
-            asset_id = asset_details['metadata']['guid']
-
+        if self.wml_client.ICP:
             TestAutoAIRemote.data_connection = DataConnection(
-                                location=DSLocation(asset_id=asset_id))
+                                location=FSLocation(path=self.data_location))
             TestAutoAIRemote.results_connection = None
 
         else: #for cloud and COS
@@ -147,10 +169,17 @@ class TestAutoAIRemote(unittest.TestCase):
                 location=S3Location(bucket=self.bucket_name,
                                     path=self.results_cos_path)
             )
-            TestAutoAIRemote.data_connection.write(data=self.data, remote_name=self.data_cos_path)
+            TestAutoAIRemote.data_connection.write(data=self.data_location, remote_name=self.data_cos_path)
 
         self.assertIsNotNone(obj=TestAutoAIRemote.data_connection)
 
+    def test_02a_read_saved_remote_data_before_fit(self):
+        TestAutoAIRemote.data = self.data_connection.read(csv_separator=self.custom_separator)
+        print("Data sample:")
+        print(self.data.head())
+        self.assertGreater(len(self.data), 0)
+
+
     #################################
     #       REMOTE OPTIMIZER        #
     #################################
@@ -163,7 +192,10 @@ class TestAutoAIRemote(unittest.TestCase):
             prediction_column='species',
             scoring=self.experiment.Metrics.ACCURACY_SCORE,
             test_size=0.15,
-            max_number_of_estimators=1
+            max_number_of_estimators=1,
+            csv_separator=self.custom_separator,
+            t_shirt_size=TShirtSize.S if self.wml_client.ICP else TShirtSize.L,
+            # autoai_pod_version=self.pod_version
         )
 
         self.assertIsInstance(self.remote_auto_pipelines, RemoteAutoPipelines,
@@ -175,10 +207,19 @@ class TestAutoAIRemote(unittest.TestCase):
         self.assertIsInstance(parameters, dict, msg='Config parameters are not a dictionary instance.')
 
     def test_05_fit_run_training_of_auto_ai_in_wml(self):
-        TestAutoAIRemote.trained_pipeline_details = self.remote_auto_pipelines.fit(
-            training_data_reference=[self.data_connection],
-            training_results_reference=self.results_connection,
-            background_mode=False)
+        failure_count = 0
+        while failure_count < 4:
+            try:
+                TestAutoAIRemote.trained_pipeline_details = self.remote_auto_pipelines.fit(
+                    training_data_reference=[self.data_connection],
+                    training_results_reference=self.results_connection,
+                    background_mode=False)
+                break
+            except AutoAIComputeError as e:
+                import time
+                failure_count += 1
+                print(f"Failure {failure_count}\n error:{e}\n\n")
+                time.sleep(10)
 
         TestAutoAIRemote.run_id = self.trained_pipeline_details['metadata']['id']
         self.assertIsInstance(self.trained_pipeline_details, dict,
@@ -187,50 +228,58 @@ class TestAutoAIRemote(unittest.TestCase):
 
         self.assertIsNotNone(self.data_connection.auto_pipeline_params,
                              msg='DataConnection auto_pipeline_params was not updated.')
-        TestAutoAIRemote.train_data, TestAutoAIRemote.holdout_data = self.remote_auto_pipelines.get_data_connections()[0].read(with_holdout_split=True)
+
+    def test_06_get_train_data(self):
+        TestAutoAIRemote.train_data = self.remote_auto_pipelines.get_data_connections()[0].read()
         print("train data sample:")
         print(self.train_data.head())
         self.assertGreater(len(self.train_data), 0)
-        print("holdout data sample:")
-        print(self.holdout_data.head())
-        self.assertGreater(len(self.holdout_data), 0)
 
-    def test_06_get_run_status(self):
+    def test_07_get_run_status(self):
         status = self.remote_auto_pipelines.get_run_status()
         self.assertEqual(status, "completed", msg="AutoAI run didn't finished successfully. Status: {}".format(status))
 
-    def test_07_get_run_details(self):
+    def test_08_get_run_details(self):
         parameters = self.remote_auto_pipelines.get_run_details()
-        # print(parameters)
+        print(parameters)
 
-    def test_08_predict_using_fitted_pipeline(self):
-        predictions = self.remote_auto_pipelines.predict(X=self.holdout_data.drop(['species'], axis=1).values[:5])
+    def test_09_predict_using_fitted_pipeline(self):
+        predictions = self.remote_auto_pipelines.predict(X=self.train_data.drop(['species'], axis=1).values[:5])
         print(predictions)
         self.assertGreater(len(predictions), 0)
 
-    def test_09_summary_listing_all_pipelines_from_wml(self):
+    def test_10_summary_listing_all_pipelines_from_wml(self):
         pipelines_details = self.remote_auto_pipelines.summary()
         print(pipelines_details)
 
-    def test_10_get_pipeline_params_specific_pipeline_parameters(self):
+    def test_11_get_pipeline_params_specific_pipeline_parameters(self):
         pipeline_params = self.remote_auto_pipelines.get_pipeline_details(pipeline_name='Pipeline_1')
-        # print(pipeline_params)
+        self.assertIsNotNone(pipeline_params)
+        self.assertIn('composition_steps', pipeline_params)
+        print(pipeline_params['composition_steps'])
+        self.assertIn('pipeline_nodes', pipeline_params)
+        print(pipeline_params['pipeline_nodes'])
+        self.assertIn('features_importance', pipeline_params)
+        print(pipeline_params['features_importance'])
+        self.assertIn('roc_curve', pipeline_params)
+        print(pipeline_params['roc_curve'])
+        self.assertIn('confusion_matrix', pipeline_params)
+        print(pipeline_params['confusion_matrix'])
+        self.assertIn('ml_metrics', pipeline_params)
+        print(pipeline_params['ml_metrics'])
 
-    def test_11_get_pipeline_params_best_pipeline_parameters(self):
+    def test_12_get_pipeline_params_best_pipeline_parameters(self):
         best_pipeline_params = self.remote_auto_pipelines.get_pipeline_details()
+        self.assertIsNotNone(best_pipeline_params)
         # print(best_pipeline_params)
 
-    def test_12_get_pipeline_load_specified_pipeline(self):
+    def test_13_get_pipeline_load_specified_pipeline(self):
         TestAutoAIRemote.lale_pipeline = self.remote_auto_pipelines.get_pipeline(
             pipeline_name='Pipeline_1')
         print(f"Fetched pipeline type: {type(self.lale_pipeline)}")
         self.assertIsInstance(self.lale_pipeline, TrainablePipeline)
-        # predictions = self.lale_pipeline.predict(
-        #     X=self.holdout_data.drop(['species'], axis=1).values[:5])  # works with autoai_libs from branch `development` (date: 9th March 20)
-        # print(predictions)
 
-
-    def test_12a_get_all_pipelines_as_lale(self):
+    def test_14_get_all_pipelines_as_lale(self):
         summary = self.remote_auto_pipelines.summary()
         print(summary)
         failed_pipelines = []
@@ -241,7 +290,7 @@ class TestAutoAIRemote(unittest.TestCase):
                 lale_pipeline = self.remote_auto_pipelines.get_pipeline(pipeline_name=pipeline_name)
                 self.assertIsInstance(lale_pipeline, TrainablePipeline)
                 predictions = lale_pipeline.predict(
-                    X=self.X.values[:1])
+                    X=self.train_data.values[:1])
                 print(predictions)
                 self.assertGreater(len(predictions), 0, msg=f"Returned prediction for {pipeline_name} are empty")
             except:
@@ -259,19 +308,21 @@ class TestAutoAIRemote(unittest.TestCase):
     #        HISTORICAL RUNS        #
     #################################
 
-    def test_13_list_historical_runs_and_get_run_ids(self):
+    def test_15_list_historical_runs_and_get_run_ids(self):
         runs_df = self.experiment.runs(filter=self.OPTIMIZER_NAME).list()
         print(runs_df)
         self.assertIsNotNone(runs_df)
         self.assertGreater(len(runs_df), 0)
 
+        runs_completed_df = runs_df[runs_df.state =='completed']
+
         from random import randint
-        TestAutoAIRemote.random_run_id = runs_df.run_id[
-            randint(len(runs_df) // 2, len(runs_df) - 1)]  # random run_id from the 2nd part of runs table
+        TestAutoAIRemote.random_run_id = runs_completed_df.run_id[
+            randint(len(runs_completed_df) // 2, len(runs_completed_df) - 1)]  # random run_id from the 2nd part of runs table
         print("Random run_id from 2nd half of df: {}".format(TestAutoAIRemote.random_run_id))
         self.assertIsNotNone(TestAutoAIRemote.random_run_id)
 
-    def test_14_get_params_of_last_historical_run(self):
+    def test_16_get_params_of_last_historical_run(self):
         run_params = self.experiment.runs.get_params(run_id=self.run_id)
         self.assertIn('prediction_type', run_params,
                       msg="prediction_type field not fount in run_params. Run_params are: {}".format(run_params))
@@ -281,17 +332,17 @@ class TestAutoAIRemote(unittest.TestCase):
                               msg="historical_optimizer is not type RemoteAutoPipelines. It's type of {}".format(
                                   type(self.historical_opt)))
 
-    def test_15_get_last_historical_pipeline_and_predict_on_historical_pipeline(self):
+    def test_17_get_last_historical_pipeline_and_predict_on_historical_pipeline(self):
         print("Getting pipeline for last run_id={}".format(self.run_id))
         summary = self.historical_opt.summary()
         pipeline_name = summary.index.values[0]
         historical_pipeline = self.historical_opt.get_pipeline(pipeline_name, astype=self.experiment.PipelineTypes.SKLEARN)
         print(type(historical_pipeline))
-        predictions = historical_pipeline.predict(self.X.values[-2:])
+        predictions = historical_pipeline.predict(self.train_data.values[-2:])
         print(predictions)
         self.assertGreater(len(predictions), 0, msg="Empty predictions")
 
-    def test_16_get_random_historical_optimizer_and_its_pipeline(self):
+    def test_18_get_random_historical_optimizer_and_its_pipeline(self):
         run_params = self.experiment.runs.get_params(run_id=self.random_run_id)
         self.assertIn('prediction_type', run_params,
                       msg="prediction_type field not fount in run_params. Run_params are: {}".format(run_params))\
@@ -301,6 +352,9 @@ class TestAutoAIRemote(unittest.TestCase):
                               msg="historical_optimizer is not type RemoteAutoPipelines. It's type of {}".format(
                                   type(historical_opt)))
         summary = self.historical_opt.summary()
+
+        self.assertGreater(len(summary), 0, msg=f"No pipelines found for optimizer with run_id = {self.random_run_id},"
+                                                f" and parameters: {run_params}")
         pipeline_name = summary.index.values[0]
         pipeline = historical_opt.get_pipeline(pipeline_name, self.experiment.PipelineTypes.SKLEARN)
         print(type(pipeline))
@@ -311,13 +365,13 @@ class TestAutoAIRemote(unittest.TestCase):
     #################################
 
     def test_20_deployment_setup_and_preparation(self):
-        if is_cp4d():
-            TestAutoAIRemote.service = WebService(wml_credentials=self.wml_credentials.copy(),
-                                                     project_id=self.project_id,
-                                                     space_id=self.space_id)
+        if self.wml_client.ICP:
+            TestAutoAIRemote.service = WebService(source_wml_credentials=self.wml_credentials.copy(),
+                                                  source_project_id=self.project_id,
+                                                  source_space_id=self.space_id)
             self.wml_client.set.default_space(self.space_id)
         else:
-            TestAutoAIRemote.service = WebService(wml_credentials=self.wml_credentials)
+            TestAutoAIRemote.service = WebService(source_wml_credentials=self.wml_credentials)
 
         delete_model_deployment(self.wml_client, deployment_name=self.DEPLOYMENT_NAME)
 
@@ -332,23 +386,115 @@ class TestAutoAIRemote(unittest.TestCase):
 
     def test_22_score_deployed_model(self):
         nb_records = 5
-        predictions = self.service.score(payload=self.holdout_data.drop(['species'], axis=1)[:nb_records])
+        predictions = self.service.score(payload=self.train_data.drop(['species'], axis=1)[:nb_records])
         print(predictions)
         self.assertIsNotNone(predictions)
         self.assertEqual(len(predictions['predictions'][0]['values']), nb_records)
 
     def test_23_list_deployments(self):
-        if is_cp4d():
-            unittest.skip("Not supported on CP4D")
+        self.service.list()
+        params = self.service.get_params()
+        print(params)
+        self.assertIsNotNone(params)
+
+    def test_24_delete_deployment(self):
+        print("Delete current deployment: {}".format(self.service.deployment_id))
+        self.service.delete()
+
+    def test_25_deployment_setup_and_preparation_different_wml(self):
+        if self.wml_client.ICP:
+            TestAutoAIRemote.service = WebService(source_wml_credentials=self.wml_credentials.copy(),
+                                                  source_project_id=self.project_id,
+                                                  source_space_id=self.space_id,
+                                                  target_wml_credentials=self.wml_credentials.copy(),
+                                                  target_project_id=self.project_id,
+                                                  target_space_id=self.space_id
+                                                  )
+            self.wml_client.set.default_space(self.space_id)
         else:
-            self.service.list()
-            params = self.service.get_params()
+            TestAutoAIRemote.service = WebService(source_wml_credentials=self.wml_credentials,
+                                                  target_wml_credentials=self.wml_credentials
+                                                  )
+
+        delete_model_deployment(self.wml_client, deployment_name=self.DEPLOYMENT_NAME)
+
+        self.wml_client.set.default_project(self.project_id) if self.wml_client.ICP else None
+
+    def test_26_deploy_pipepline_by_pipeline_name_different_wml(self):
+        TestAutoAIRemote.service.create(
+            model='Pipeline_4',
+            deployment_name=self.DEPLOYMENT_NAME,
+            experiment_run_id=self.run_id
+        )
+
+    def test_27_score_deployed_model_different_wml(self):
+        nb_records = 5
+        predictions = self.service.score(payload=self.train_data.drop(['species'], axis=1)[:nb_records])
+        print(predictions)
+        self.assertIsNotNone(predictions)
+        self.assertEqual(len(predictions['predictions'][0]['values']), nb_records)
+
+    def test_28_list_deployments_different_wml(self):
+        self.service.list()
+        params = self.service.get_params()
+        print(params)
+        self.assertIsNotNone(params)
+
+    def test_29_delete_deployment_different_wml(self):
+        print("Delete current deployment: {}".format(self.service.deployment_id))
+        self.service.delete()
+
+    #########################
+    #  Batch deployment
+    #########################
+
+    def test_30_batch_deployment_setup_and_preparation(self):
+        if not self.wml_client.ICP:
+            self.skipTest("Batch deployment is not available on Cloud")
+        else:
+            TestAutoAIRemote.service_batch = Batch(self.wml_credentials,
+                                                   source_space_id=self.space_id)
+
+            self.assertIsInstance(self.service_batch, Batch, msg="Deployment is not of Batch type.")
+            self.assertIsInstance(self.service_batch._source_workspace, WorkSpace, msg="Workspace set incorrectly.")
+            self.assertEqual(self.service_batch.id, None, msg="Deployment ID initialized incorrectly")
+            self.assertEqual(self.service_batch.name, None, msg="Deployment name initialized incorrectly")
+
+    def test_32__deploy__batch_deploy_best_computed_pipeline_from_autoai_on_wml(self):
+        if not self.wml_client.ICP:
+            self.skipTest("Batch deployment is not available on Cloud")
+        else:
+            self.service_batch.create(
+                experiment_run_id=self.remote_auto_pipelines._engine._current_run_id,
+                model=self.lale_pipeline,
+                deployment_name=self.DEPLOYMENT_NAME+'BATCH')
+
+    def test_33_score_batch_deployed_model(self):
+        if not self.wml_client.ICP:
+            self.skipTest("Batch deployment is not available on Cloud")
+        else:
+            scoring_params = self.service_batch.run_job(
+                payload=self.train_data.drop(columns=['species']),
+                background_mode=False)
+            print(scoring_params)
+
+    def test_34_list_batch_deployments(self):
+        if not self.wml_client.ICP:
+            self.skipTest("Batch deployment is not available on Cloud")
+        else:
+            deployments = self.service_batch.list()
+            print(deployments)
+            params = self.service_batch.get_params()
             print(params)
             self.assertIsNotNone(params)
 
-    def test_24_delete_deployment(self):
-        if is_cp4d():
-            unittest.skip("Not supported on CP4D")
+    def test_35_delete_deployment_batch(self):
+        if not self.wml_client.ICP:
+            self.skipTest("Batch deployment is not available on Cloud")
         else:
             print("Delete current deployment: {}".format(self.service.deployment_id))
-            self.service.delete()
+            self.service_batch.delete()
+            self.assertEqual(self.service_batch.id, None, msg="Deployment ID deleted incorrectly")
+            self.assertEqual(self.service_batch.name, None, msg="Deployment name deleted incorrectly")
+            self.assertEqual(self.service_batch.scoring_url, None,
+                             msg="Deployment scoring_url deleted incorrectly")
