@@ -8,16 +8,18 @@ from tarfile import open as open_tar
 from time import sleep
 from typing import Dict, Union, Tuple, List, TYPE_CHECKING
 from warnings import warn
+from functools import wraps
 
 import requests
 from watson_machine_learning_client.wml_client_error import ApiRequestFailure
 
 from .enums import RunStateTypes
-from .errors import MissingPipeline, AutoAIComputeError, DataFormatNotSupported, LibraryNotCompatible, \
-    CannotInstallLibrary, CannotDownloadTrainingDetails
+from .errors import (MissingPipeline, AutoAIComputeError, DataFormatNotSupported, LibraryNotCompatible,
+                     CannotInstallLibrary, CannotDownloadTrainingDetails, CannotDownloadWMLPipelineDetails,
+                     VisualizationFailed)
 
 if TYPE_CHECKING:
-    from io import BytesIO
+    from io import BytesIO, BufferedIOBase
     from pandas import DataFrame
     from collections import OrderedDict
     from sklearn.pipeline import Pipeline
@@ -39,12 +41,17 @@ __all__ = [
     'try_import_autoai_libs',
     'try_import_tqdm',
     'try_import_xlrd',
+    'try_import_graphviz',
     'prepare_cos_client',
     'create_model_download_link',
     'create_summary',
     'prepare_auto_ai_model_to_publish_notebook',
+    'get_node_and_runtime_index',
     'download_experiment_details_from_file',
-    'prepare_model_location_path'
+    'prepare_model_location_path',
+    'download_wml_pipeline_details_from_file',
+    'init_cos_client',
+    'check_graphviz_binaries'
 ]
 
 
@@ -120,31 +127,29 @@ def fetch_pipelines(run_params: dict,
         # note: iterate over all computed pipelines
         for pipeline in run_params['entity']['status'].get('metrics', []):
 
-            # note: take care only for global_output pipelines
-            if pipeline['context']['phase'] == 'global_output':
-                # note: fetch and create model paths from file system
-                model_path = pipeline['context']['intermediate_model']['location']['model']
-                # --- end note
+            # note: fetch and create model paths from file system
+            model_path = pipeline['context']['intermediate_model']['location']['model']
+            # --- end note
 
-                # note: populate available pipeline names
-                if pipeline_name is None:  # checking all pipelines
-                    model_paths.append(model_path)
-                    pipelines_names.append(
-                        f"Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}")
+            # note: populate available pipeline names
+            if pipeline_name is None:  # checking all pipelines
+                model_paths.append(model_path)
+                pipelines_names.append(
+                    f"Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}")
 
-                    # note: check dependencies for estimators
-                    check_pipeline_nodes(pipeline=pipeline)
+                # note: check dependencies for estimators
+                check_pipeline_nodes(pipeline=pipeline)
 
-                # checking only chosen pipeline
-                elif pipeline_name == f"Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}":
-                    model_paths.append(model_path)
-                    pipelines_names = [f"Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}"]
+            # checking only chosen pipeline
+            elif pipeline_name == f"Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}":
+                model_paths.append(model_path)
+                pipelines_names = [f"Pipeline_{pipeline['context']['intermediate_model']['name'].split('P')[-1]}"]
 
-                    # note: check dependencies for estimators
-                    check_pipeline_nodes(pipeline=pipeline)
+                # note: check dependencies for estimators
+                check_pipeline_nodes(pipeline=pipeline)
 
-                    break
-                # --- end note
+                break
+            # --- end note
 
         if load_pipelines:
             # Disable printing to suppress warning from ai4ml
@@ -479,6 +484,10 @@ def prepare_auto_ai_model_to_publish(
 
         # note: workaround for the cloud deployment, storing model, schema and pipeline definition on COS
         path = f"{run_params['entity']['results_reference']['location']['path']}/{run_params['metadata']['guid']}/"
+
+        # note: change model location in pipeline-mode.json to new one
+        modify_pipeline_model_json(data_location=wml_pipeline_definition_name, model_path=path)
+
         cos_client.meta.client.upload_file(
             Filename=temp_model_name,
             Bucket=bucket,
@@ -500,6 +509,27 @@ def prepare_auto_ai_model_to_publish(
         return path
 
 
+def modify_pipeline_model_json(data_location: str, model_path: str) -> None:
+    """
+    Change the location of KB model in pipeline-model.json
+
+    Parameters
+    ----------
+    data_location: str, required
+        pipeline-model.json data local path
+
+    model_path: str, required
+        Path to KB model stored in COS.
+    """
+    with open(data_location, 'r') as f:
+        data = json.load(f)
+
+    data['pipelines'][0]['nodes'][-1]['parameters']['output_model']['location'] = f"{model_path}model.pickle"
+
+    with open(data_location, 'w') as f:
+        f.write(json.dumps(data))
+
+
 def init_cos_client(connection: dict) -> 'resource':
     """Initiate COS client for further usage."""
     from ibm_botocore.client import Config
@@ -591,7 +621,8 @@ class RunDetailsGenerator:
             else:
                 raise AutoAIComputeError(self.run_id, reason=f"Error: {message}")
 
-        elif state == RunStateTypes.COMPLETED or message != self.message:
+        # note: additional checks, OBM produces similar completed message state, we need to finish only when KB finished
+        elif state == RunStateTypes.COMPLETED and 'Training job' in message and 'completed' in message:
             if self.finished:
                 raise StopIteration
             else:
@@ -669,11 +700,11 @@ def try_import_autoai_libs():
              f"This is the needed dependency for pipeline model refinery, we will try to install it now...")
 
         try:
-            check_call([executable, "-m", "pip", "install", "autoai_libs>=1.10.12"])
+            check_call([executable, "-m", "pip", "install", "autoai_libs>=1.11.0"])
 
         except Exception as e:
             raise CannotInstallLibrary(value_name=e,
-                                       reason="autoai_libs>=1.10.12 failed to install. Please install it manually.")
+                                       reason="autoai_libs>=1.11.0 failed to install. Please install it manually.")
 
 
 def try_import_tqdm():
@@ -714,13 +745,33 @@ def try_import_xlrd():
                                        reason="xlrd==1.2.0 failed to install. Please install it manually.")
 
 
-def try_load_dataset(buffer: 'BytesIO', sheet_name: str = 0, separator: str = ',') -> Union['DataFrame', 'OrderedDict']:
+def try_import_graphviz():
+    """
+    Check if graphviz package is installed in local environment, if not, just download and install it.
+    """
+    try:
+        import graphviz
+
+    except ImportError:
+        warn(f"\"graphviz\" package is not installed. "
+             f"This is the needed dependency for visualizing data join graph, we will try to install it now...")
+
+        try:
+            check_call([executable, "-m", "pip", "install", "graphviz==0.14"])
+
+        except Exception as e:
+            raise CannotInstallLibrary(value_name=e,
+                                       reason="graphviz==0.14 failed to install. Please install it manually.")
+
+
+def try_load_dataset(buffer: Union['BytesIO', 'BufferedIOBase'], sheet_name: str = 0, separator: str = ',') -> Union[
+    'DataFrame', 'OrderedDict']:
     """
     Load data into a pandas DataFrame from BytesIO object.
 
     Parameters
     ----------
-    buffer: BytesIO, required
+    buffer: Union['BytesIO', 'BufferedIOBase'], required
         Buffer with bytes data.
 
     sheet_name: str, optional
@@ -737,7 +788,7 @@ def try_load_dataset(buffer: 'BytesIO', sheet_name: str = 0, separator: str = ',
 
     try:
         buffer.seek(0)
-        data = read_csv(buffer, sep=separator)
+        data = read_csv(buffer, sep=separator, encoding='utf-8')
 
     except Exception as e1:
         try:
@@ -975,6 +1026,33 @@ def create_summary(details: dict, scoring: str) -> 'DataFrame':
         return pipelines
 
 
+def get_node_and_runtime_index(node_name: str, optimizer_config: dict) -> Tuple[int, int]:
+    """Find node index from node name in experiment parameters."""
+    node_number = None
+    runtime_number = None
+
+    for i, node in enumerate(optimizer_config['entity']['document']['pipelines'][0]['nodes']):
+        if node_name == 'kb' and (node.get('id') == 'kb' or node.get('id') == 'automl'):
+            node_number = i
+            break
+
+        elif node_name == 'obm' and node.get('id') == 'obm':
+            node_number = i
+            break
+
+    for i, runtime in enumerate(optimizer_config['entity']['document']['runtimes']):
+        if node_name == 'kb' and (runtime.get('id') == 'kb' or runtime.get('id') == 'automl' or
+                                  runtime.get('id') == 'autoai'):
+            runtime_number = i
+            break
+
+        elif node_name == 'obm' and runtime.get('id') == 'obm':
+            runtime_number = i
+            break
+
+    return node_number, runtime_number
+
+
 def download_experiment_details_from_file(result_client_and_connection: Tuple['DataConnection', 'resource']) -> dict:
     """Try to download training details from user COS."""
 
@@ -991,6 +1069,25 @@ def download_experiment_details_from_file(result_client_and_connection: Tuple['D
     return details
 
 
+def download_wml_pipeline_details_from_file(result_client_and_connection: Tuple['DataConnection', 'resource']) -> dict:
+    """Try to download wml pipeline details from user COS."""
+
+    try:
+        path = result_client_and_connection[0].location._model_location.split('model.pickle')[0]
+        path = f"{path}pipeline-model.json"
+
+        file = result_client_and_connection[1].Object(
+            result_client_and_connection[0].location.bucket,
+            path).get()
+
+        details = json.loads(file['Body'].read())
+
+    except Exception as e:
+        raise CannotDownloadWMLPipelineDetails('', reason=f"Error: {e}")
+
+    return details
+
+
 def prepare_model_location_path(model_path: str) -> str:
     """
     To be able to get best pipeline after computation we need to change model_location string to global_output.
@@ -1005,3 +1102,20 @@ def prepare_model_location_path(model_path: str) -> str:
         path = f"{path}data/kb/global_output/"
 
     return path
+
+
+def check_graphviz_binaries(f):
+    @wraps(f)
+    def _f(*method_args, **method_kwargs):
+        from graphviz.backend import ExecutableNotFound
+        try:
+            output = f(*method_args, **method_kwargs)
+
+        except ExecutableNotFound as e:
+            raise VisualizationFailed(
+                reason=f"Cannot perform visualization with graphviz. Please make sure that you have Graphviz binaries "
+                       f"installed in your system. Please follow this guide: https://www.graphviz.org/download/")
+
+        return output
+
+    return _f
