@@ -7,9 +7,23 @@
 #
 ################################################################################
 
-from watson_machine_learning_client.log_util import get_logger
+from watson_machine_learning_client.utils.log_util import get_logger
 import sys
 
+__all__ = [
+    "WMLClientError",
+    "MissingValue",
+    "MissingMetaProp",
+    "NotUrlNorUID",
+    "NoWMLCredentialsProvided",
+    "ApiRequestFailure",
+    "UnexpectedType",
+    "ForbiddenActionForPlan",
+    "NoVirtualDeploymentSupportedForICP",
+    "MissingArgument",
+    "WrongEnvironmentVersion"
+]
+
 
 class WMLClientError(Exception):
     def __init__(self, error_msg, reason = None):
@@ -56,6 +70,20 @@ class ForbiddenActionForPlan(WMLClientError):
     def __init__(self, operation_name, expected_plans, actual_plan):
         WMLClientError.__init__(self, 'Operation \'{}\' is available only for {} plan, while this instance has \'{}\' plan.'.format(operation_name, ('one of {} as'.format(expected_plans) if len(expected_plans) > 1 else '\'{}\''.format(expected_plans[0])) if type(expected_plans) is list else '\'{}\''.format(expected_plans), actual_plan))
 
+
 class NoVirtualDeploymentSupportedForICP(MissingValue):
     def __init__(self, reason = None):
-        MissingValue.__init__(self, 'No Virtual deployment supported for ICP', reason)
\ No newline at end of file
+        MissingValue.__init__(self, 'No Virtual deployment supported for ICP', reason)
+
+
+class MissingArgument(WMLClientError, ValueError):
+    def __init__(self, value_name, reason=None):
+        WMLClientError.__init__(self, f"Argument: {value_name} missing.", reason)
+
+
+class WrongEnvironmentVersion(WMLClientError, ValueError):
+    def __init__(self, used_version, environment_name, supported_versions):
+        WMLClientError.__init__(self, "Version used in credentials not supported in this environment",
+                                reason=f"Version {used_version} isn't supported in "
+                                       f"{environment_name} environment, "
+                                       f"select from {supported_versions}")
\ No newline at end of file
