@@ -347,14 +347,14 @@ class WMLResource:
             return 1
         else:
             return 0
-
+    # TODO:  Fix extra "done" getting printed
     def _list(self, values, header, limit, default_limit, sort_by='CREATED'):
+        # TODO :  Find out why extra "done" message getting printed.
         if sort_by is not None and sort_by in header:
             column_no = header.index(sort_by)
             values = sorted(values, key=lambda x: x[column_no], reverse=True)
 
         from tabulate import tabulate
-
         if limit is None:
             table = tabulate([header] + values[:default_limit])
             print(table)
@@ -368,10 +368,9 @@ class WMLResource:
         _DEFAULT_LIST_LENGTH = 50
         ##For CP4D, check if either spce or project ID is set
         href = self._href_definitions.get_wsd_asset_search_href(asset_type)
+        
         if limit is None:
-            data = {
-                "query": "*:*"
-            }
+            data = {"query": "*:*"}
         else:
             self._validate_type(limit, u'limit', int, False)
             data = {
@@ -587,7 +586,11 @@ class WMLResource:
                 new_el = {'metadata': metadata_val,
                           'entity': response_data['entity']
                           }
-
+                if 'frameworkName' in new_el['entity']:
+                     new_el['entity'].pop('frameworkName')
+                     new_el['entity'].pop('frameworkVersion')
+                if 'framework_runtimes' in new_el['entity']:
+                    new_el['entity'].pop('framework_runtimes')
                 return new_el
             else:
                 return response_data
